#=
# The code produces BDIO files for md_info (see OrderedDict md_info) and Wilson observables (see OrderedDict obs_w)
# The BDIO files are saved in path_bdio_md and path_bdio_w respectivily. Each BDIO file contains information about a given ensemble.
# Plateaux for different ensembles are defined in txt files (path_plat_w)
# The dat files must be located in the following way
    path_data_w/ensemble -> *mesons.dat (Wilson)
    path_data_w/ensemble -> *ms.dat
    path_rw/ensemble -> *.ms1.dat
    path_md/ensemble -> *.pbp.dat
# Example: path_rw/H400 contains H400r001.ms1.dat and H400r002.ms1.dat
# The target value of phi4 can be modified
# Masses and decay constants are in terms of the reference scale t0
# See read_bdio.jl for more information
=#
using Base: String
using LaTeXStrings: length
using OrderedCollections
using juobs, BDIO, DelimitedFiles, ADerrors, LaTeXStrings

#================== PATHS + INFO ==================#
const path_data_w = "/Users/alessandroconigli/Lattice/data/aux_obs_data/wilson"

const path_plat_w = "/Users/alessandroconigli/Lattice/data/aux_obs_data/plat_wilson.txt"
const path_rw = "/Users/alessandroconigli/Lattice/data/aux_obs_data/rwf"
const path_md = "/Users/alessandroconigli/Lattice/data/aux_obs_data/md"

const ens_list = ["D200"] # ["H101", "H102r002", "H400", "N202", "N200", "N203", "N300", "J303"]
const path_bdio_md = "/Users/alessandroconigli/Lattice/data/bdio_charm/md"
const path_bdio_w = "/Users/alessandroconigli/Lattice/data/bdio_charm/wilson"

const phi4_target = uwreal([1.117, 0.012], "phi4_target")
const rwf = true

#========= INCLUDES ==========#
include("./types.jl")
include("./tools.jl")
include("./const.jl")

#======== GET ENSEMBLE INFORMATION FROM DATABASE ==========#
ensinfo = Vector{EnsInfo}(undef, length(ens_list))
for i in 1:length(ens_list)
    ens = ens_list[i]
    try
        ensinfo[i]= EnsInfo(ens, ens_db[ens], trunc_db[ens])
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database.
        Please check the ensemble id or update the database")
    end
end
#t0 [2.86, 3.659, 5.164, 8.595]
wpmm = Dict{String, Vector{Float64}}()
wpmm["H101"] = [-1.0, 4.0, -1.0, 14.0*2.86]
wpmm["H102r002"] = [-1.0, 4.0, -1.0, 14.0*2.86]
wpmm["H105"] = [-1.0, 4.0, -1.0, 14.0*2.86]
wpmm["H400"] = [-1.0, 4.0, -1.0, 14.0*3.659]
wpmm["N202"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["N200"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["N203"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["N300"] = [-1.0, 4.0, -1.0, 14.0*8.595]
wpmm["J303"] = [-1.0, 4.0, -1.0, 14.0*8.595]
##
#======== ANALYSIS ==========#
gen_mulist = Vector{Vector{Vector{Float64}}}(undef, length(ens_list))

md_info = Vector(undef, length(ensinfo))
obs_tm_shifted = Vector(undef, length(ensinfo))

#obs
obs_w = Vector(undef, length(ensinfo))
for (k, ens) in enumerate(ensinfo)    
    # t0
    t0_ens, YW, W = get_t0(path_data_w, ens, rw=rwf, path_rw=path_rw, pl=true)
    
    # WILSON correlators
    pp_w, pp_wW, _ = get_corr(path_data_w, ens, "G5", "G5", rw=rwf, path_rw=path_rw, legacy=true)
    pp_w_d1, _, _ = get_corr(path_data_w, ens, "G5_d1", "G5_d1", rw=rwf, path_rw=path_rw, legacy=true)
    pp_w_d2, _, _ = get_corr(path_data_w, ens, "G5_d2", "G5_d2", rw=rwf, path_rw=path_rw, legacy=true)

    # WILSON masses
    m_w = get_meff_w(pp_w, ens, pl=true)
    kappa_list = getfield.(pp_w, :kappa)
    mll_w = get_ll_w(kappa_list, m_w, ens.deg)
    mls_w = ens.deg ? mll_w : get_ls_w(kappa_list, m_w, ens.deg)

    phi2 = 8 * t0_ens * mll_w^2
    phi4 = ens.deg ? 8 * t0_ens * 1.5 * mll_w^2 : 8 * t0_ens * (0.5 * mll_w^2 + mls_w^2)
    
    # MASS SHIFTS  
    dSdm = read_pbp(path_md, ens.id)

    ppll = get_ll_w(kappa_list, pp_w, ens.deg)
    ppllW = get_ll_w(kappa_list, pp_wW, ens.deg)
    ppll_d1 = get_ll_w(kappa_list, pp_w_d1, ens.deg)
    ppll_d2 = get_ll_w(kappa_list, pp_w_d2, ens.deg)

    dphi4_s1, dphi4_s2 = md_sea(phi4, dSdm, YW, W) .+ md_sea(phi4, dSdm, ppllW, W)
    dphi2_s1, dphi2_s2 = md_sea(phi2, dSdm, YW, W) .+ md_sea(phi2, dSdm, ppllW, W)

    dphi4_v1, dphi4_v2 = md_val(phi4, ppll, [ppll_d1, ppll_d2])
    dphi2_v1, dphi2_v2 = md_val(phi2, ppll, [ppll_d1, ppll_d2])

    if !ens.deg
        ppls = get_ls_w(kappa_list, pp_w, ens.deg)
        pplsW = get_ls_w(kappa_list, pp_wW, ens.deg)
        ppls_d1 = get_ls_w(kappa_list, pp_w_d1, ens.deg)
        ppls_d2 = get_ls_w(kappa_list, pp_w_d2, ens.deg)

        aux1, aux2 = md_val(phi4, ppls, [ppls_d1, ppls_d2])
        dphi4_v1 = dphi4_v1 + aux1
        dphi4_v2 = dphi4_v2 + aux2
        dphi4_s1, dphi4_s2 =  (dphi4_s1, dphi4_s2) .+ md_sea(phi4, dSdm, pplsW, W)
    end
    # Dict: Deltam, phi2, phi4, t0 (shifted)
    deltam = (phi4_target - phi4) / (2*dphi4_s1 + dphi4_s2 + dphi4_v1 + dphi4_v2)
    dt0_s1, dt0_s2 = md_sea(t0_ens, dSdm, YW, W)
    t0_ens_shifted = t0_ens + deltam * (2 * dt0_s1 + dt0_s2) 
    phi2_shifted = phi2 + deltam * (2*dphi2_s1 + dphi2_s2 + dphi2_v1 + dphi2_v2)
    phi4_shifted = phi4 + deltam * (2*dphi4_s1 + dphi4_s2 + dphi4_v1 + dphi4_v2)
    
    md_info[k] = OrderedDict(
        "deltam" => deltam,
        "t0_shifted" => t0_ens_shifted,
        "phi2_shifted" => phi2_shifted,
        "phi4_shifted" => phi4_shifted
    )
    obs_w[k] = OrderedDict(
        "mll" => sqrt(8 * t0_ens) * mll_w,
        "mls" => sqrt(8 * t0_ens) * mls_w,
        "t0" => t0_ens,
        "phi2" => phi2,
        "phi4" => phi4
    )
end
## WRITE BDIO
# MD INFO
for (k, obs) in enumerate(md_info) #loop over ensembles
    ens = ensinfo[k].id
    p = joinpath(path_bdio_md, string(ens, ".bdio"))
    fb = BDIO_open(p, "w", ens)
    uinfo = 0
    for (key, value) in obs #loop over observables
        write_uwreal(value, fb, uinfo)
        uinfo += 1
    end
    BDIO_close!(fb)
end
# WILSON
for (k, obs) in enumerate(obs_w) #loop over ensembles
    ens = ensinfo[k].id
    p = joinpath(path_bdio_w, string(ens, ".bdio"))
    fb = BDIO_open(p, "w", ens)
    uinfo = 0
    for (key, value) in obs #loop over observables
        write_uwreal(value, fb, uinfo)
        uinfo += 1
    end
    BDIO_close!(fb)
end