function read_data(path::String, ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    p = joinpath(path, ens)
    rep = filter(x->occursin("mesons.dat", x), readdir(p, join=true))
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons(rep[1], g1, g2, legacy=legacy, id=ens)) : (return read_mesons(rep, g1, g2, legacy=legacy, id=ens))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_pbp(path::String, ens::String)
    p = joinpath(path, ens)
    rep = filter(x->occursin("pbp.dat", x), readdir(p, join=true))
    if length(rep)!=0
        return read_md.(rep)
    else
        error("pbp.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_rw(path::String, ens::String)
    p = joinpath(path, ens)
    rep = filter(x->occursin("ms1.dat", x), readdir(p, join=true))
    if length(rep)!=0
        try
            length(rep) == 1 ? (return read_ms1(rep[1])) : (return read_ms1.(rep)) 
        catch
            length(rep) == 1 && length(rep)!=0 ? (return read_ms1(rep[1], v="1.4")) : (return read_ms1.(rep, v="1.4")) 
        end
    else
        error("ms1.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function read_t0(path::String, ens::String, dtr::Int64)
    p = joinpath(path, ens)
    rep = filter(x->occursin("ms.dat", x), readdir(p, join=true))
    if length(rep)!=0
        length(rep) == 1 ? (return read_ms(rep[1], dtr=dtr, id=ens)) : (return read_ms.(rep, dtr=dtr, id=ens)) 
    else
        error("ms.dat file not found for ensemble ", ens, " in path ", p)
    end
end
function get_corr(path::String, ens::EnsInfo, g1::String="G5", g2::String="G5"; rw::Bool=false, legacy::Bool=false, path_rw::String="")
    if path_rw == ""
        p_rw = path
    else
        p_rw = path_rw
    end

    aux = read_data(path, ens.id, g1, g2, legacy=legacy)
    if !isnothing(ens.trunc)
        truncate_data!(aux, ens.trunc)
    end
    if !rw
        obs = corr_obs.(aux, L=ens.L, info=true)
        return (getindex.(obs, 1), getindex.(obs, 2))
    else
        obs = corr_obs.(aux, L=ens.L, rw=read_rw(p_rw, ens.id), info=true)
        return (getindex.(obs, 1), getindex.(obs, 2), getindex.(obs, 3))
    end
end
function get_t0(path::String, ens::EnsInfo; rw::Bool=false, pl::Bool=false, path_rw::String="")
    if path_rw == ""
        p_rw = path
    else
        p_rw = path_rw
    end
    data = read_t0(path, ens.id, ens.dtr)
    if !isnothing(ens.trunc)
        truncate_data!(data, ens.trunc)
    end
    plat = select_plateau_t0(ens) 
    if !rw
        t0 = comp_t0(data, plat, L=ens.L, pl=pl, info=true) 
        return (getindex(t0, 1), getindex(t0, 2))
    else
        t0 = comp_t0(data, plat, L=ens.L, pl=pl, rw=read_rw(p_rw, ens.id), info=true)
        return (getindex(t0, 1), getindex(t0, 2), getindex(t0, 3))
    end
end
function get_mu(mu_list::Vector{Vector{Float64}}, deg::Bool)
    mu_sorted = unique(sort(minimum.(mu_list)))

    mul = mu_sorted[1]
    deg ? mus = 0.0 : mus = mu_sorted[2]
    muh = unique(maximum.(mu_list))
    muh = filter(x-> x > mul && x > mus, muh)

    return mul, mus, muh
end
function get_kappa(kappa_list::Vector{Vector{Float64}}, deg::Bool)
    kappa_sorted = unique(sort(maximum.(kappa_list), rev=true))

    kappal = kappa_sorted[1]
    deg ? kappas = 0.0 : kappas = kappa_sorted[2]

    return kappal, kappas
end
function get_ll(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] == mu[2] #l-l
            return obs[k]
        end
    end
end
function get_ll_w(kappa_list, obs::Vector, deg::Bool)
    kappal, kappas = get_kappa(kappa_list, deg)
    for k = 1:length(kappa_list)
        kappa = kappa_list[k]
        if kappal in kappa && kappa[1] == kappa[2] #l-l
            return obs[k]
        end
    end
end
function get_ls(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mus in mu  #l-l
            return obs[k]
        end
    end
end
function get_ls_w(kappa_list, obs::Vector, deg::Bool)
    kappal, kappas = get_kappa(kappa_list, deg)
    for k = 1:length(kappa_list)
        kappa = kappa_list[k]
        if kappal in kappa && kappas in kappa  #l-l
            return obs[k]
        end
    end
end
function get_lh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_lh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu) #l-h
            push!(obs_lh, obs[k])
        end
    end
    return obs_lh
end
function get_ss(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] == mu[2] #s-s
            return obs[k]
        end
    end
end
function get_sh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_sh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] != mu[2] && !(mul in mu)#s-h
            push!(obs_sh, obs[k])
        end
    end
    return obs_sh
end

function get_hh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_hh = Vector{uwreal}(undef,0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        for i =1:length(muh)
            if muh[i] in mu && mu[1] == mu[2]#h-h
                push!(obs_hh, obs[k])
            end
        end
    end
    return obs_hh
end
function get_meff_tm(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau_tm(ens, mu_list)
    println(plat)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_meff_vec(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau_vec(ens, mu_list)
    println(plat)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_meff_w(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    kappa_list  = getfield.(obs, :kappa)

    plat = select_plateau_w(ens, kappa_list)
    println(plat)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_f_tm(obs::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo; pl::Bool=false, wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    mu_list = getfield.(obs, :mu)
    plat = select_plateau_tm(ens, mu_list)
    return dec_const_pcvc.(obs, plat, m, pl=pl, wpm=wpm)
end
function select_plateau_t0(ensinfo::EnsInfo)
    ens = ensinfo.id
    f = readdlm(path_plat_w)
    head  = String.(f[:,1])
    delim = findall(x->occursin("#", x), head)
    edelim = findfirst(x->occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)
    del = delim[cdelim]+1 : delim[cdelim+1]-1
    head_ = String.(f[del, 1])
    plat_ = Int64.(f[del, 2:3])
    plat = Vector{Int64}(undef, 2)
    n = findfirst(x-> occursin("t0", x), head_)
    plat[1] = plat_[n,1]
    plat[2] = plat_[n,2]
    return plat
end
function select_plateau_tm(ensinf::EnsInfo, mu_list)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat_tm)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end
function select_plateau_vec(ensinf::EnsInfo, mu_list)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat_tm)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh_vec", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll_vec", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls_vec", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh_vec", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss_vec", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh_vec", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end
function select_plateau_w(ensinf::EnsInfo, kappa_list)
    ens = ensinf.id
    deg = ensinf.deg
    kappal, kappas = get_kappa(kappa_list, deg)
    f = readdlm(path_plat_w)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    for k = 1:length(kappa_list)
        kappa = kappa_list[k]
        if kappal in kappa && kappa[1] == kappa[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif kappal in kappa && kappas in kappa#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )

        elseif kappas in kappa && kappa[1] == kappa[2] && !(kappal in kappa)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end


function match_muc(muh, m_lh, m_sh, target)
    M = (2/3 .* m_lh .+ 1/3 .* m_sh) 
    par, chi2exp = lin_fit(muh, M)
    muh_target = x_lin_fit(par, target)
    return muh_target
end

function aic_weight(aic)
    w_aux = exp.(-1/2 * aic /minimum(aic)) #/minimum(aic) -> +0.5*mininum(aic)
    w_aux1=filter(x->x<10000, w_aux)
    index = findall(x->x<10000, w_aux)
    norm = sum(w_aux1)
    w = w_aux1 ./ norm
    return w, index 
end
function aic_weight(c::Cat)
    aic = c.aic
    w_aux = exp.(-1/2 * aic /minimum(aic)) #/minimum(aic) -> +0.5*mininum(aic)
    w_aux1=filter(x->x<10000, w_aux)
    index = findall(x->x<10000, w_aux)
    norm = sum(w_aux1)
    w = w_aux1 ./ norm
    return w, index
end
function aic_syst(aic, fit_res)
    w,index = aic_weight(aic)
    aux1 = sum( w .* fit_res[index].^2)
    aux2 = sum( w .* fit_res[index])^2
    return sqrt(abs(value(aux1 - aux2)))
end
function aic_syst(c::Cat)
    w,index = aic_weight(c.aic)
    aux1 = sum( w .* c.fit_res[index].^2)
    aux2 = sum( w .* c.fit_res[index])^2
    return sqrt(abs(value(aux1 - aux2)))
end

function aic_model_ave(aic, fit_res, wpm::Union{Nothing, Dict}=nothing)
    w, index = aic_weight(aic)
    res = sum( w.* fit_res[index])
    isnothing(wpm) ? uwerr(res) : uwerr(res, wpm)
    return res 
end
function aic_model_ave(c::Cat, wpm::Union{Nothing, Dict}=nothing)
    w, index = aic_weight(c)
    res = sum( w.* getfield(c, :fit_res)[index])
    isnothing(wpm) ? uwerr(res) : uwerr(res, wpm)
    return res 
end
function category_av(cat_arr::Vector{Cat})
    best_res = aic_model_ave.(cat_arr)
    syst = aic_syst.(cat_arr)
    tot_err = err.(best_res).^2 .+ syst.^2
    w_aux = 1 ./ (tot_err)
    w = w_aux ./ sum(w_aux)
    println(findmax(w)[2])
    aux1 = sum( w .* best_res.^2)
    aux2 = sum( w .* best_res)^2
    sys1 = sqrt(abs(value(aux1 - aux2)))
    av = sum( w .* best_res)

    return av, sys1
end
function category_av_tot(cat_arr::Vector{Cat})
    fit_res = vcat(getfield.(cat_arr, :fit_res)...)
    aic = vcat(getfield.(cat_arr, :aic)...)
    best = aic_model_ave(aic, fit_res)
    syst = aic_syst(aic, fit_res)
    return best, syst
end

function syst_av(syst_err::Vector{Float64})
    aux = sum(syst_err.^2)
    return sqrt(aux)
end

function cat_obs(obs::Vector{Vector{uwreal}})
    N = length(obs[1])
    if !all(length.(obs) .== N)
        error("Dimension mismatch")
    end

    r = getindex.(obs, 1)
    for i = 2:N
        r = vcat(r, getindex.(obs, i))
    end
    return r
end

function Base.:*(x::uwreal, y::Array{Any})
    N = size(y, 1)
    return fill(x, N) .* y
end  
function Base.:*(x::uwreal, y::Array{Float64})
    N = size(y, 1)
    return fill(x, N) .* y
end
function Base.:*(x::uwreal, y::Array{uwreal})
    N = size(y, 1)
    return fill(x, N) .* y
end
function Base.:+(x::uwreal, y::Vector{uwreal})
    N = size(y, 1)
    return fill(x, N) .+ y
end
function Base.:+(x::Float64, y::Vector{Float64})
    N = size(y, 1)
    return fill(x, N) .+ y
end
function Base.:+(x::Float64, y::Vector{uwreal})
    N = size(y, 1)
    return fill(x, N) .+ y
end
function Base.:-(x::Float64, y::Vector{Any})
    N = size(y, 1)
    return fill(x, N) .- y
end
function Base.:-(x::Float64, y::Vector{Float64})
    N = size(y, 1)
    return fill(x, N) .- y
end
function Base.:-(x::Float64, y::Vector{uwreal})
    N = size(y, 1)
    return fill(x, N) .- y
end