#################################
# This file was created by Alessandro Conigli - 2022
# The pourpose of this code is to read BDIO files 
# and perform chiral+continuum extrapolation.
#  
##################################

using Base: String
using LaTeXStrings: length
using OrderedCollections
using juobs, BDIO, DelimitedFiles, ADerrors, LaTeXStrings, PyPlot, Statistics

#============= SET UP VARIABLES ===========#
rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] =  false
rcParams["mathtext.fontset"]  = "cm"
rcParams["font.size"] =12
rcParams["axes.labelsize"] =20
rcParams["axes.titlesize"] = 18

plt.rc("text", usetex=false) # set to true once you install latex
#rcParams.update(PyPlot.matplotlib.mpl.rcParamsDefault)
#================== PATHS ==================#
# data by ale (carfull with the read_BDIO that you include)

const path_bdio = "/Users/alessandroconigli/Lattice/data/bdio_charm" 
#const path_bdio = "/Users/alessandroconigli/Lattice/data/bdio_charm/from_gevp/tm_shifted/ale"
const path_bdio_tm = joinpath(path_bdio, "tm_shifted")
const path_bdio_md = joinpath(path_bdio, "md")

# data by javier (carful with the read_BDIO that you include. do not use the one in this folder)
# const path_bdio = "/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/analysis_with_javier_data/bdio" #PATH to BDIO folder
# const path_bdio_tm = joinpath(path_bdio, "tm_shifted")
# const path_bdio_md = joinpath(path_bdio, "md")
# by reading javier data you have deactivated the reading of vector decay constants

const path_plot = "/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/plots"

#========= INCLUDES ==========#
include("./const.jl")
include("./types.jl")
include("./tools.jl")
include("./func_comb.jl")
# include("/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/analysis_with_javier_data/read_bdio.jl")
include("./read_bdio.jl") # this is the one to include with data by ale 
include("./gevp_plotter.jl")



#======== GET ENSEMBLE INFORMATION FROM DATABASE ==========#
enslist = filter(x -> occursin(".bdio", x), readdir(path_bdio_tm))
Nens = length(enslist)
ensname = [enslist[k][1:end-5] for k = 1:Nens]
ensinfo = Vector{EnsInfo}(undef, length(ensname))
for (i, ens) in enumerate(ensname)
    try
        ensinfo[i]= EnsInfo(ens, ens_db[ens], trunc_db[ens])
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database.
        Please check the ensemble id or update the database")
    end
end
wpmm = Dict{String, Vector{Float64}}()
wpmm["H101"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["H102r002"] = [-1.0, 2.0, -1.0, -1.0]
wpmm["H400"]     = [-1.0, 1.5, -1.0, -1.0]
wpmm["N202"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N200"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N203"]     = [-1.0, 2.0, -1.0, -1.0]
wpmm["N300"]     = [-1.0, 1.5, -1.0, -1.0]
wpmm["J303"]     = [-1.0, 2.0, -1.0, -1.0]


# READ MASSES + DECAY CONSTANTS
muh = Vector{Vector{uwreal}}(undef, Nens)

mlh = Vector{Vector{uwreal}}(undef, Nens)
msh = Vector{Vector{uwreal}}(undef, Nens)
mhh = Vector{Vector{uwreal}}(undef, Nens)

mlh_v = Vector{Vector{uwreal}}(undef, Nens)
msh_v = Vector{Vector{uwreal}}(undef, Nens)
mhh_v = Vector{Vector{uwreal}}(undef, Nens)

flh = Vector{Vector{uwreal}}(undef, Nens)
fsh = Vector{Vector{uwreal}}(undef, Nens)
fhh = Vector{Vector{uwreal}}(undef, Nens)

flh_v = Vector{Vector{uwreal}}(undef, Nens)
fsh_v = Vector{Vector{uwreal}}(undef, Nens)
fhh_v = Vector{Vector{uwreal}}(undef, Nens)

phih = Vector{Dict{String, Vector{uwreal}}}(undef, Nens)
a28t0 = Vector{uwreal}(undef, Nens)
phi2 = Vector{uwreal}(undef, Nens)
phi4 = Vector{uwreal}(undef, Nens)

for (k, ens) in enumerate(enslist)
    p = joinpath(path_bdio_tm, ens)
    mlh[k] = read_BDIO(p, "tm", "mlh")
    msh[k] = read_BDIO(p, "tm", "msh")
    mhh[k] = read_BDIO(p, "tm", "mhh")
    mll = read_BDIO(p, "tm", "mll")[1]
    mls = read_BDIO(p, "tm", "mls")[1]

    mlh_v[k] = read_BDIO(p, "tm", "mlh_star")
    msh_v[k] = read_BDIO(p, "tm", "msh_star")
    mhh_v[k] = read_BDIO(p, "tm", "mhh_star")

    flh[k] = read_BDIO(p, "tm", "flh")
    fsh[k] = read_BDIO(p, "tm", "fsh")
    fhh[k] = read_BDIO(p, "tm", "fhh")

    flh_v[k] = read_BDIO(p, "tm", "flh_star") .* za.(ensinfo[k].beta) 
    fsh_v[k] = read_BDIO(p, "tm", "fsh_star") .* za.(ensinfo[k].beta)
    fhh_v[k] = read_BDIO(p, "tm", "fhh_star") .* za.(ensinfo[k].beta)

    phih_fa = (2 * mlh[k] + msh[k]) / 3
    phih_sfa = (3 * phih_fa + 6 * mlh_v[k] + 3 * msh_v[k]) / 12
    phih_eta = mhh[k]
    phih[k] = Dict(
        "phih_fa" => phih_fa,
        "phih_sfa" => phih_sfa,
        "phih_eta" => phih_eta
    )
    muh[k] = read_BDIO(p, "tm", "muh") 
    zp_aux = fill(ZP(ensinfo[k].beta), length(muh[k]))
    muh[k] ./= zp_aux

    p = joinpath(path_bdio_md, ens)
    a28t0[k] = 1 / ( 8 * read_BDIO(p, "md", "t0_shifted")[1])
    phi2[k] = mll^2
    phi4[k] = mls^2 + 0.5 * mll^2
end

## X VARIABLE [a^2/(8t0) phi_2 phi_h]

x_fa = [a28t0 phi2 getindex.(getindex.(phih, "phih_fa"), 1)]
x_sfa = [a28t0 phi2 getindex.(getindex.(phih, "phih_sfa"), 1)]
x_eta = [a28t0 phi2 getindex.(getindex.(phih, "phih_eta"), 1)]

for i = 2:length(muh[1])
    x_fa = vcat(x_fa, [a28t0 phi2 getindex.(getindex.(phih, "phih_fa"), i)])
    x_sfa = vcat(x_sfa, [a28t0 phi2 getindex.(getindex.(phih, "phih_sfa"), i)])
    x_eta = vcat(x_eta, [a28t0 phi2 getindex.(getindex.(phih, "phih_eta"), i)])
end

# PHYSICAL VALUES
phih_eta_ph =  M[5] * t0_ph / hc
phih_fa_ph =  (2 * M[1] +  M[3]) * t0_ph / (3 * hc)
phih_sfa_ph =  (2 * M[1] +  M[3] + 6 * M[2] + 3 * M[4]) * t0_ph / (12 * hc)
phi2_ph = (t0_ph * mpi_ph / hc)^2 #isospin symmetric mpi = 134.8 MeV

#============= CREATE MASS_CAT =================#
# these are categories for runs with 3 value of the charm mass - hard coded

# s_obs_mass = ["muc", "m_etac", "m_jpsi", "mD", "mDs", "mD_star", "mDs_star"]
# obs = cat_obs.([muh, mhh, mhh_v, mlh, msh, mlh_v, msh_v])
# s_match = ["fl_ave", "etac", "sp_fl_ave"]
# match = [x_fa, x_eta, x_sfa]
# phih_ph = [phih_fa_ph, phih_eta_ph, phih_sfa_ph]

# this is for general number of charm masses 
# keep in mind that the order of the data is differente: above it was for each mh all the ensembles,
# here it is for each ensembles all the mh values. No differences in fits but xdata and ydata need to be correctly ordered

s_obs_mass = ["muc", "m_etac", "m_jpsi", "mD", "mDs", "mD_star", "mDs_star"]
obs = cat_obs_vcat.([muh, mhh, mhh_v, mlh, msh, mlh_v, msh_v])
s_match = ["fl_ave", "etac", "sp_fl_ave"]

[uwerr.(obs[i]) for i in 1:length(obs)]

ens_id = getindex.(ensembles.(obs[1]),2)
a28t0_tot = Vector{Vector{uwreal}}(undef, length(ensinfo))
phi2_tot  = Vector{Vector{uwreal}}(undef, length(ensinfo))
for (k, ens) in enumerate(ensinfo)
    idx = findall(x->x==ens.id, ens_id)
    a28t0_tot[k] = fill(a28t0[k], length(idx)) 
    phi2_tot[k]  = fill(phi2[k], length(idx))
end

x_fa  = [vcat(a28t0_tot...) vcat(phi2_tot...) vcat(getindex.(phih, "phih_fa")...) ]
x_eta = [vcat(a28t0_tot...) vcat(phi2_tot...) vcat(getindex.(phih, "phih_eta")...) ]
x_sfa = [vcat(a28t0_tot...) vcat(phi2_tot...) vcat(getindex.(phih, "phih_sfa")...) ]
match = [x_fa, x_eta, x_sfa]
phih_ph = [phih_fa_ph, phih_eta_ph, phih_sfa_ph]

# creating mass categories 
mass_cat = Vector{Vector{Cat}}(undef, 0)
for (i, o) in enumerate(obs)
    aux = Vector{Cat}(undef, 0)
    for (j, m) in enumerate(match)
        push!(aux, Cat(o, m, phih_ph[j], string(s_obs_mass[i], "_", s_match[j])))
    end
    push!(mass_cat, aux)
end

##
#============= FIT MESON MASSES =============#
@info("Fitting meson masses for charm quark matching")
# mD
fit_over_cat!(mass_cat[4], f, phi2_ph, n_param, weight_dof=2, correlated_fit=false, Mrat=uwreal(1.0), small_sample=true)
# mDs
fit_over_cat!(mass_cat[5], f, phi2_ph, n_param, weight_dof=2, correlated_fit=false, Mrat=uwreal(1.0), small_sample=true)
# mD_star
fit_over_cat!(mass_cat[6], f, phi2_ph, n_param, weight_dof=2, correlated_fit=false, Mrat=uwreal(1.0), small_sample=true)
# mDs_star
fit_over_cat!(mass_cat[7], f, phi2_ph, n_param, weight_dof=2, correlated_fit=false, Mrat=uwreal(1.0), small_sample=true)
##
s = [L"$\sqrt{8t_0} Z^{tm}_M \mu_c$", L"$\sqrt{8t_0}M_{\eta^{(c)}_c}$", L"$\sqrt{8t_0}M_{J/\psi}$",
L"$\sqrt{8t_0}M_D$", L"$\sqrt{8t_0}M_{D_s}$", L"$\sqrt{8t_0}M_{D^{*}}$", L"$\sqrt{8t_0}M_{D^{*}_s}$"]
idx_cat = 4 # 4,5
for i in 2:2
    #plot_cat(mass_cat[idx_cat][i], s[idx_cat] , save=false)    
    #plot_chi2_vs_exp(mass_cat[idx_cat][i], label_cutoff ) #, p=path_plot)
    plot_chi_charm(mass_cat[idx_cat][i], vcat(f_aux...), s[idx_cat], phi2_ph, ensinfo, mrat=uwreal(1.0), p=path_plot)
    #plot_cl_charm(mass_cat[idx_cat][i],  vcat(f_aux...), s[idx_cat], phi2_ph, ensinfo, mrat=uwreal(1.0))#, p=path_plot)
end
##
#============= FIT CHARM MASS ==============#

@info("Fitting charm quark mass")
fit_over_cat!(mass_cat[1], f, phi2_ph, n_param, weight_dof=2, correlated_fit=false, Mrat=Mrat, small_sample=true  )

##  plot charm
for i in 1:3
    plot_cat(mass_cat[1][i], L"$M_c^{\mathrm{RGI}}(N_f=3)\mathrm{MeV}$" , save=false)
    plot_chi2_vs_exp(mass_cat[1][i], label_cutoff)#, p=path_plot)
    plot_chi_charm(mass_cat[1][i], vcat(f_aux...),  L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, mrat=Mrat,  p=path_plot)
    plot_cl_charm(mass_cat[1][i], vcat(f_aux...), L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo,   mrat=Mrat)#,    p=path_plot)
end
close("all")
## plot for multiple categories
plot_hist_tot([mass_cat[1][1], mass_cat[1][2]], L"$M^\mathrm{RGI}_c\,(N_f=3)\,(\mathrm{MeV})$" , p=path_plot)
plot_all_cl_charm([mass_cat[1][2], mass_cat[1][1], mass_cat[1][3]], vcat(f_aux...),L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, mrat=Mrat, n=[0,0,1] )#, p=path_plot) 
plot_cat_tot(mass_cat[1][1:2], L"$M^\mathrm{RGI}_c\,(N_f=3)\,(\mathrm{MeV})$" , save=true)
##

#============== CREATE DECAY CAT =============#
s_obs_f = ["fD", "fDs", "f_etac"]
s_match = ["fl_ave", "etac", "sp_fl_ave"]
phih_ph = [phih_fa_ph, phih_eta_ph, phih_sfa_ph]

## for uncorrelated fit the orginal order ens1...ensN,ens1...ensN,ens1...ensN is fine
obs = cat_obs.([flh, fsh, fhh])
match = [x_fa, x_eta, x_sfa]

## for correlated fits the order must be different: ens11..ens13, ens21...ens23,...,ensN1..ensN3
obs = [vcat(flh...), vcat(fsh...), vcat(fhh...)]
[uwerr.(obs[i]) for i in 1:length(obs)]

# preparing xdata for fitting in correct order
ens_id = getindex.(ensembles.(obs[1]),2)
a28t0_tot = Vector{Vector{uwreal}}(undef, length(ensinfo))
phi2_tot  = Vector{Vector{uwreal}}(undef, length(ensinfo))
for (k, ens) in enumerate(ensinfo)
    idx = findall(x->x==ens.id, ens_id)
    a28t0_tot[k] = fill(a28t0[k], length(idx)) 
    phi2_tot[k]  = fill(phi2[k], length(idx))
end
x_fa  = [vcat(a28t0_tot...) vcat(phi2_tot...) vcat(getindex.(phih, "phih_fa")...) ]
x_eta = [vcat(a28t0_tot...) vcat(phi2_tot...) vcat(getindex.(phih, "phih_eta")...) ]
x_sfa = [vcat(a28t0_tot...) vcat(phi2_tot...) vcat(getindex.(phih, "phih_sfa")...) ]

match = [x_fa, x_eta, x_sfa]

## create categories 
f_cat = Vector{Vector{Cat}}(undef, 0)
for (i, o) in enumerate(obs)
    aux = Vector{Cat}(undef, 0)
    for (j, m) in enumerate(match)
        push!(aux, Cat(o, m, phih_ph[j], string(s_obs_f[i], "_", s_match[j])))
    end
    push!(f_cat, aux)
end

##########################
## TEST ON fD/fDs
#f_ratio_cat = Vector{Cat}(undef, 0)
#for (j,m) in enumerate(match)
#    push!(f_ratio_cat, Cat(obs[2]./obs[1], m, phih_ph[j], string("fds_over_fd", "_", s_match[j]) ))
#end
obs_ratio = obs[2]./obs[1]
idx_no_sym = findall(x->x!=1.0, value.(obs_ratio))
obs_ratio = obs_ratio[idx_no_sym]
#obs_ratio[1:3] .+= 0.015
xratio = x_fa[idx_no_sym, 1:3]
f_ratio_cat = Cat(obs_ratio, xratio, phih_ph[1], string("fds_over_fd") )
ratio_model(x,p) = 1 .+ p[1].*(3. .*phi2_sym .- 3. .* x[:,2]) .+ p[2].*x[:,1]
ratio_model_aux = (x,p) -> 1. + p[1]*(3. *phi2_sym - 3. * x[:,2]) + p[2]*x[:,1]
par, chi_vs_chi2exp = fit_routine(ratio_model, value.(f_ratio_cat.x_tofit), f_ratio_cat.obs, 2)
xx = [0.0 phi2_ph]

ress = ratio_model_aux(xx, par)[1]
push!(f_ratio_cat.fit_res, ress )
push!(f_ratio_cat.fit_param, par)
## plot ratio
idx_ens_nosim = findall( x-> x==false, getfield.(ensinfo, :deg))
obs_ratio .+= 0.01

plot_chi_fds_ratio(f_ratio_cat, ratio_model_aux, L"$f_{D_s}/f_D$", phi2_ph, ensinfo[idx_ens_nosim], p=path_plot)
##
function plot_chi_fds_ratio(c::Cat, ff::Function, ylab::LaTeXString, phi2_ph::uwreal, ensinfo::Vector{EnsInfo}; p::Union{String, Nothing}=nothing )

    color = ["orange", "darkgreen", "magenta", "navy"]
    fmt = ["^", "v", "<", ">"]
    a_sp = [0.087, 0.077, 0.065, 0.05]

    phi2_max =   phi2_sym#maximum(value.(phi2))
    upar = getfield(c, :fit_param)[1]
    phih_2_use = value(c.phih_ph)
    res = getfield(c, :fit_res)[1]
    xx = [fill(1e-8, 100) Float64.(range(0.01, phi2_max, length=100)) ]
    yy = ff(xx, upar) 

    #Project one value of muc_i (i=1,2,3)
    n_unique = unique(i-> c.x_tofit[i,2], 1:length(c.x_tofit[:,2]))
    p2 = c.x_tofit[n_unique, 2]
    a2 = c.x_tofit[n_unique, 1]
    pc = c.x_tofit[n_unique, 3]
    x = [a2 p2 pc]
    x2 = [a2 p2 fill(phih_2_use, length(p2))] #physical value phih
    y = ( c.obs[n_unique] - ff(x, upar) + ff(x2, upar)) #Projection
    uwerr.(yy)
    uwerr(res)
    uwerr.(y)
    #Plotting
    figure(figsize=(10,6))
    plot(phi2_sym, 1., marker="X", ms=10, color="black", label= L"$M_\pi=M_K$")
    fill_between(xx[:,2], value.(yy)+ err.(yy), value.(yy)-err.(yy), alpha=0.3, color="royalblue") #CL extrapolation (band)
    errorbar(value(phi2_ph), value(res), err(res), fmt="s", color="red", label=L"$\phi_2 = \phi^\mathrm{phys}_2$", capsize=5, ms=12)#CL extrapolation
    for (k,b) in enumerate(sort(unique(getfield.(ensinfo, :beta))))
        n_ = findall(x->x.beta == b, ensinfo)
        a2_aux = mean(value.(a2[n_]))
        errorbar(value.(x[n_,2]), value.(y[n_]), err.(y[n_]), fmt=fmt[k], label=string(L"$a \approx$", a_sp[k], " fm"), color=color[k], capsize=5, ms=12, mfc="none")#projected datapoints
        #dashed lines
        #xxx = [fill(a2_aux, 100) Float64.(range(0.0, phi2_max, length=100)) fill(value(c.phih_ph), 100)]
        #yyy = ff(xxx, upar)
        #plot(xxx[:,2], value.(yyy), ls="--", color=color[k])#dashed lines 
    end
    ylabel(ylab, size=22)
    xlabel(L"$\phi_2$", size=22)
    legend(ncol=2, fontsize=16)
    #ylim(3.02, 3.18)
    tight_layout()
    grid(ls="dashed")
    if !isnothing(p)
        t = "fit_fds_ratio_" * c.info * ".pdf"
        savefig(joinpath(p, t))
        close()
    else
        display(gcf())
        close("all")
    end

end
##

#=========== FIT DECAY CONSTANT ==============#

## FIT DEC. CONSTS fD + fDs (LIN)

for i = 1:3
    f1 = f_cat[1][i]
    f2 = f_cat[2][i]
    for j = 1:length(f_f_lin1)
        for k = 1:length(f_f_lin1[j])
            println(" j is :", j, " k is equal: ", k)

            par, chi_vs_chi2exp = fit_routine([f_f_lin1[j][k], f_f_lin2[j][k]], [value.(f1.x_tofit), value.(f2.x_tofit)], 
            [f1.obs, f2.obs], f_n_param_comb[j], correlated_fit=false)
                    
        
            #aux1 = par[1] + par[2] * phi2_ph + par[3] / sqrt(f1.phih_ph)
            #aux2 = par[1] + par[2] * (3*phi2_sym - 2*phi2_ph) + par[3] / sqrt(f1.phih_ph)

            x = [0.0 phi2_ph f1.phih_ph]
            aux1 = f_f_aux1[1][1](x, par)[1]
            aux2 = f_f_aux2[1][1](x, par)[1]
            doff = sum(length.([f1.obs, f2.obs])) - f_n_param_comb[j]
            push!(f1.fit_res, aux1)
            push!(f2.fit_res, aux2)
            push!(f1.fit_param, par)
            push!(f2.fit_param, par)
            push!(f1.chi2_vs_exp, chi_vs_chi2exp)
            push!(f2.chi2_vs_exp, chi_vs_chi2exp)
            push!(f1.n_param, f_n_param_comb[j])
            push!(f2.n_param, f_n_param_comb[j])
            push!(f1.dof, doff)
            push!(f2.dof, doff)
            # use first 2 lines for uncorrelated fits
            AIC = chi_vs_chi2exp * doff + 2 * f_n_param_comb[j] + ( 2*f_n_param_comb[j]^2 + 2*f_n_param_comb[j]) / (length(f1.obs)- f_n_param_comb[j] -1)
            push!(f1.aic, AIC)
            push!(f2.aic, AIC)
            # push!(f2.aic, chi_vs_chi2exp  - 2 * doff)
            # push!(f1.aic, chi_vs_chi2exp  - 2 * doff)
        
        end
    end
end

## FIT DEC. CONSTS fD + fDs (LIN + CLOGS)

for i = 1:3
    f1 = f_cat[1][i]
    f2 = f_cat[2][i]
    for j = 1:length(f_f_lin1_clog)
        for k = 1:length(f_f_lin1_clog[j])
            println(" j is :", j, " k is equal: ", k)
            par, chi2_vs_chi2exp = fit_routine([f_f_lin1_clog[j][k], f_f_lin2_clog[j][k]], [value.(f1.x_tofit), value.(f2.x_tofit)], 
            [f1.obs, f2.obs], f_n_param_comb_clog[j], correlated_fit=false)


            x = [0.0 phi2_ph f1.phih_ph]
            aux1 = f_f_aux1_clog[1][1](x, par)[1]
            aux2 = f_f_aux2_clog[1][1](x, par)[1]
            doff = sum(length.([f1.obs, f2.obs])) - f_n_param_comb_clog[j]

            push!(f1.fit_res, aux1)
            push!(f2.fit_res, aux2)
            push!(f1.fit_param, par)
            push!(f2.fit_param, par)
            push!(f1.chi2_vs_exp, chi2_vs_chi2exp)
            push!(f2.chi2_vs_exp, chi2_vs_chi2exp)
            push!(f1.n_param, f_n_param_comb_clog[j])
            push!(f2.n_param, f_n_param_comb_clog[j])
            push!(f1.dof, doff)
            push!(f2.dof, doff)
            # use first 2 lines for uncorrelated fits
            push!(f1.aic, chi2_vs_chi2exp * doff + 2 * f_n_param_comb_clog[j])
            push!(f2.aic, chi2_vs_chi2exp * doff + 2 * f_n_param_comb_clog[j])
            # push!(f2.aic, chi2_vs_chi2exp  - 2 * doff)
            # push!(f1.aic, chi2_vs_chi2exp  - 2 * doff)
         
        end


    end

end


## 
# ffD  =  vcat(f_f_aux1...)
# ffDs =  vcat(f_f_aux2...)

ffD =  vcat(f_f_aux1_clog...)
ffDs =  vcat(f_f_aux2_clog...)
# ffD =  vcat([vcat(f_f_aux1...), vcat(f_f_aux1_clog...)]...) # lin+log
# ffDs =  vcat([vcat(f_f_aux2...), vcat(f_f_aux2_clog...)]...) # lin+log

s = [L"$\sqrt{8t_0}f_D$", L"$\sqrt{8t_0}f_{D_s}$", L"$\sqrt{8t_0}f_{\eta^{(c)}_c}$"]
for i=1:3
    plot_cl_charm( f_cat[1][i], ffD, s[1], phi2_ph, ensinfo,  dec=true)#,    p=path_plot) 
    plot_cl_charm( f_cat[2][i], ffDs, s[2], phi2_ph, ensinfo, dec=true)#,   p=path_plot)
    plot_chi_charm(f_cat[1][i], ffD, s[1], phi2_ph, ensinfo,  dec=true)#,   p=path_plot) 
    plot_chi_charm(f_cat[2][i], ffDs, s[2], phi2_ph, ensinfo, dec=true)#,   p=path_plot)
    plot_cat(f_cat[1][i], s[1] , save=false) 
    plot_cat(f_cat[2][i], s[2] , save=false)
    #plot_chi2_vs_exp(f_cat[1][i], label_cutoff)# , p=path_plot)
    #plot_chi2_vs_exp(f_cat[2][i], label_cutoff)#, p=path_plot)
    # plot_chi2_vs_exp_clog(f_cat[1][i], label_cutoff , p=path_plot)
    # plot_chi2_vs_exp_clog(f_cat[2][i], label_cutoff , p=path_plot)
end
close("all")
##
plot_cl_charm(f_cat[1][1], vcat(f_f_aux1...),L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, dec=true, n=1 ) # 35 nice
plot_chi_charm(f_cat[1][2], vcat(f_f_aux1...),  L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, dec=true, n=1) #, p=path_plot)


#############
## RESULTS ##
#############
#t0 [2.86, 3.659, 5.164, 8.595]
wpmm2 = Dict{String, Vector{Float64}}()
wpmm2["H101"] = [-1.0, 1.5, -1.0, -1.0]
wpmm2["H102r002"] = [-1.0, 2.0, -1.0, -1.0]
wpmm2["H400"] = [-1.0, 4.0, -1.0, -1.0]
wpmm2["N202"] = [-1.0, 1.0, -1.0, 14.0*5.146]
wpmm2["N200"] = [-1.0, 2.0, -1.0, -1.0]
wpmm2["N203"] = [-1.0, 2.0, -1.0, -1.0]
wpmm2["N300"] = [-1.0, 1.5, -1.0, 14.0*8.595]
wpmm2["J303"] = [-1.0, 2.0, -1.0, -1.0]


#============ CHARM MASS =============#
# category av fl_ave + etac
for (k, c) in enumerate([mass_cat[1]])
    o, sys = category_av_tot([c[1],c[2]])
    o_p = o * hc / t0_ph
    sys_p = sys * hc / value(t0_ph)
    uwerr(o, wpmm2)
    uwerr(o_p, wpmm2)
    println(s_obs_mass[k], "\t $o +/- $sys")
    println(s_obs_mass[k], "\t $o_p +- $sys_p")
    println(s_obs_mass[k], details(o_p))
end
# fl-av / sp-fl-av / etac
for i = 1:3
    for (k, c) in enumerate([mass_cat[1]])
        o = aic_model_ave(c[i])
        sys = aic_syst(c[i])
        uwerr(o, wpmm2)
        println(s_obs_mass[k], "\t $o +/- $sys")
        
    end
    println()
end

##
#============ DECAY CONSTANTS ==============#
for (k, c) in enumerate(f_cat[1:2])
    o, sys = category_av_tot([c[1], c[2]])
    o_p = o * hc / value(t0_ph)
    sys_p = sys * hc / value(t0_ph)
    uwerr(o, wpmm2)
    uwerr(o_p, wpmm2)
    #uwerr(o)
    #uwerr(o_p)
    println(s_obs_f[k], "\t$o +/- $sys")
    println(s_obs_f[k], "\t $o_p +- $sys_p")
    println(s_obs_f[k], details(o_p))
    push!(store, o_p)

end
# fl-av / sp-fl-av / etac
##
for i = 1:3
    for (k, c) in enumerate(f_cat[1:2])
        o = aic_model_ave(c[i])
        sys = aic_syst(c[i])
        uwerr(o, wpmm2)
        println(s_obs_f[k], "\t $o +/- $sys")
    end
    println()
end




##







#######################
### EXTRA PLOTS
#######################




ens_tot = getindex.(ensembles.(obs[1]),2)
color = ["orange", "darkgreen", "magenta", "navy", "yellow", "gray", "black", "lightgreen"]
for (k,ens) in enumerate(ensname)
    idx = findall(x->x==ens, ens_tot)
    y_muc = obs[1][idx]
    x_muc = x_sfa[idx,3]
    lin_model(x,p) = p[1] .+ x.*p[2]
    par, chi2 = juobs.fit_routine(lin_model, value.(x_muc), y_muc, 2)
    uwerr.(x_muc)
    plot(value.(x_muc), lin_model(value.(x_muc), value.(par)), color=color[k])
    errorbar(value.(x_muc), xerr=err.(x_muc), value.(y_muc), yerr=err.(y_muc), color=color[k], fmt="^", label="$(ens)")
end
xlabel(L"$\phi_H^{sp-fl}$")
ylabel(L"$\sqrt{8t_0}\mu_c^R$")
legend()
tight_layout()
savefig(joinpath(path_plot,"mc_heavy_dep.pdf"))
display(gcf())
close("all")










################
###############
## TEST WITH CORRELATED FITS 

ydat = f_cat[1][1].obs
uwerr.(ydat)
aux_id = getindex.(ensembles.(ydat),2) 
val_map = tuple.(ydat,aux_id )
##
covar_set  = Vector{Matrix{Float64}}(undef, length(unique(aux_id)) )
for (k,id) in enumerate(unique(aux_id))
    idx = findall(x-> x==id, getindex.(val_map,2) )
    fluc_aux = mchist.(ydat[idx], fill(id, length(idx)))
    fluc = mapreduce(permutedims, vcat, fluc_aux)'
    println(size(fluc)[1])
    covar_set[k] = fluc' * fluc ./ size(fluc)[1]^2
end
##
covar_set[1]
err(ydat[1])^2

function covar_multi_id(obs::Vector{uwreal}; wpm::Union{Dict{Int64,Vector{Float64}},Dict{String,Vector{Float64}}, Nothing}=nothing)
    isnothing(wpm) ? uwerr.(obs) :  [uwerr(yaux, wpm) for yaux in obs]
    id_set = getindex.(ensembles.(obs), 2) # hardcoded assuming that ens label is always the second 
    #covar_set  = Vector{Matrix{Float64}}(undef, length(unique(id_set)) )
    
    cov_tot = zeros(length(id_set), length(id_set))
    count = 1
    for (k,id) in enumerate(unique(id_set))
        idx = findall(x-> x==id, id_set )
        fluc_aux = mchist.(obs[idx], fill(id, length(idx)))
        fluc = mapreduce(permutedims, vcat, fluc_aux)'
        covar_set = fluc' * fluc ./ size(fluc)[1]^2
        dim_tmp = size(covar_set,1)
        cov_tot[count:dim_tmp+count-1, count:dim_tmp+count-1] = covar_set
        count +=dim_tmp
    end
    return cov_tot
end
##


ydat= mass_cat[1][2].obs
test = covar_multi_id(ydat)
aa = inv(test)
aa = (test + test')/2
issymmetric(aa)
isposdef(aa)

evals = Diagonal(test)^(-0.5)
bb = evals * test * evals
##
imshow(bb, cmap="hot")
title("covariance")
colorbar()
display(gcf())
close("all")
##

dim = sum(size(test[i],1) for i in 1:length(test) )
cov_tot = zeros(dim, dim)
count = 1
for (i, covar) in enumerate(test)
    dim_tmp = size(test[i],1)
    println(dim_tmp)
    cov_tot[count:dim_tmp+count-1, count:dim_tmp+count-1] = covar
    count+=dim_tmp
end


##
