#################################
# This file was created by Alessandro Conigli - 2022
# The pourpose of this code is to compute twisted mass 
# observables from raw correlators stored in mesons.dat files.
# The analysis is performed at the level of a single ensemble 
# and results are then stored in BDIO files 
# It extracts:
#  - charm quark mass 
#  - ll hl hh pseudo and vector masses 
#  - ll hl hh pseudo and vector leptonic decays
#  
#  The analysis is performed by solving a GEVP 
##################################
using Base: String
using LaTeXStrings: length
using OrderedCollections
using juobs, BDIO, DelimitedFiles, ADerrors, LaTeXStrings, PyPlot

#============= SET UP VARIABLES ===========#
rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] =  false
rcParams["mathtext.fontset"]  = "cm"
rcParams["font.size"] =10
rcParams["axes.labelsize"] =22
rcParams["axes.titlesize"] = 18

plt.rc("text", usetex=false) # set to true once you install latex

#============ PATHS & INFO =================#
# path to data 
const path_data_w  = "/Users/alessandroconigli/Lattice/data/aux_obs_data/wilson"
const path_data_tm = "/Users/alessandroconigli/Lattice/data/charm_full_Dirac"
# path to plateau
const path_plat_w       = "/Users/alessandroconigli/Lattice/data/aux_obs_data/plat_wilson.txt"
const path_plat_tm_mps  = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_mps.txt"
const path_plat_tm_mvec = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_mvec.txt"
const path_plat_tm_dps  = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_dps.txt"
const path_plat_tm_dvec = "/Users/alessandroconigli/Lattice/gevp-automated-analysis/plat/plat_tm_dvec.txt"
# path to aux obs 
const path_rw = "/Users/alessandroconigli/Lattice/data/aux_obs_data/rwf"
const path_md = "/Users/alessandroconigli/Lattice/data/aux_obs_data/md"
# path to bdio
const path_bdio_md         = "/Users/alessandroconigli/Lattice/data/bdio_charm/md"
const path_bdio_tm         = "/Users/alessandroconigli/Lattice/data/bdio_charm/tm"
const path_bdio_tm_shifted = "/Users/alessandroconigli/Lattice/data/bdio_charm/tm_shifted"
const path_plot = "/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/plots"

# ensembles to analyse 
const ens_list = ["H102r002"]
# reweighting and mass shift flags
const rwf = true
const mass_shift = true 
const tau = 3 # gevp shift parameter
const TSM = false # set whether TSM is used or not 

@warning("\nTSM FLAG MODE: FALSE\n")

#=============== INCLUDES ===============#

include("types.jl")
include("tools.jl")
include("const.jl")
include("read_bdio.jl")

#=============== ENSEMBLE INFO FROM DATABASE ==================#
ensinfo = Vector{EnsInfo}(undef, length(ens_list))
for i in 1:length(ens_list)
    ens = ens_list[i]
    try
        ensinfo[i]= EnsInfo(ens, ens_db[ens], trunc_db[ens])
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database.
        Please check the ensemble id or update the database")
    end
end
wpmm = Dict{String, Vector{Float64}}()
wpmm["H101"] = [-1.0, 4.0, -1.0, 14.0*2.86]
wpmm["H102r002"] = [-1.0, 4.0, -1.0, 14.0*2.86]
wpmm["H105"] = [-1.0, 4.0, -1.0, 14.0*2.86]
wpmm["H400"] = [-1.0, 4.0, -1.0, 14.0*3.659]
wpmm["N202"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["N200"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["N203"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["D200"] = [-1.0, 4.0, -1.0, 14.0*5.164]
wpmm["N300"] = [-1.0, 4.0, -1.0, 14.0*8.595]
wpmm["J303"] = [-1.0, 4.0, -1.0, 14.0*8.595]
##
#=============== ANALYSIS ===============#

gen_mulist = Vector{Vector{Vector{Float64}}}(undef, length(ens_list))

# obs storage  
obs_tm = Vector(undef, length(ensinfo))
if mass_shift
    deltam = Vector(undef, length(ensinfo))
    obs_tm_shifted = Vector(undef, length(ensinfo))
end

# compute observables 
for (k,ens) in enumerate(ensinfo)
    # t0
    t0_ens, YW, W = get_t0(path_data_w, ens, rw=rwf, path_rw=path_rw)
    # tm correlators  
    pp_tm, pp_tmW, _ = get_corr(path_data_tm, ens,"G5", "G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    
    a1a1_tm, _, _   = get_corr(path_data_tm, ens, "G1G5", "G1G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    a2a2_tm, _, _   = get_corr(path_data_tm, ens, "G2G5", "G2G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    a3a3_tm, _, _   = get_corr(path_data_tm, ens, "G3G5", "G3G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    
    pa1_tm, _, _    = get_corr(path_data_tm, ens, "G5", "G1G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    pa2_tm, _, _    = get_corr(path_data_tm, ens, "G5", "G2G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    pa3_tm, _, _    = get_corr(path_data_tm, ens, "G5", "G3G5", rw=rwf, path_rw=path_rw, tsm=TSM)
    
    
    mu_list = gen_mulist[k] = getfield.(pp_tm, :mu)

    # gevp matrices for ps and vec masses
    mat_tm1  = comp_mat_multigamma(ens, pp_tm, pa1_tm, a1a1_tm)
    mat_tm2  = comp_mat_multigamma(ens, pp_tm, pa2_tm, a2a2_tm)
    mat_tm3  = comp_mat_multigamma(ens, pp_tm, pa3_tm, a3a3_tm)
    # gevp matrices for ps and vec decays
    mat_dec_ps   = comp_mat(ens, pp_tm, tau)
    mat_dec_vec1 = comp_mat(ens, a1a1_tm, tau)
    mat_dec_vec2 = comp_mat(ens, a2a2_tm, tau)
    mat_dec_vec3 = comp_mat(ens, a3a3_tm, tau)

    # gevp meson masses 
    m_tm_ps, m_tm_vec1 =  gevp_mass(mat_tm1, path_plat_tm_mps, path_plat_tm_mvec, t0=3, pl=true)
    _, m_tm_vec2       =  gevp_mass(mat_tm2, path_plat_tm_mps, path_plat_tm_mvec, t0=3, pl=false)
    _, m_tm_vec3       =  gevp_mass(mat_tm3, path_plat_tm_mps, path_plat_tm_mvec, t0=3, pl=false)
    m_tm_vec = (m_tm_vec1 .+ m_tm_vec2 .+ m_tm_vec3 ) ./ 3.
    # pion mass
    pion_plat = select_plateau(ens, mu_list, path_plat_tm_mps)[1]
    #mll_g = m_tm_ps[1]
    #m_tm_ps[1] = mll_f = juobs.meff(pp_tm[1], pion_plat, pl=true)
    #test = meff.(pp_tm,select_plateau(ens, mu_list, path_plat_tm_mps), pl=true)

    #  decay constants  from gevp
    # dec_ps, data    = gevp_dec(mat_dec_ps,   m_tm_ps,   path_plat_tm_dps,  t0=3, n=1, pl=true)
    # dec_v1, datav1  = gevp_dec(mat_dec_vec1, m_tm_vec1, path_plat_tm_dvec, t0=3, n=1, pl=true, pseudo=false)
    # dec_v2, datav2  = gevp_dec(mat_dec_vec2, m_tm_vec2, path_plat_tm_dvec, t0=3, n=1, pl=true, pseudo=false)
    # dec_v3, datav3  = gevp_dec(mat_dec_vec3, m_tm_vec3, path_plat_tm_dvec, t0=3, n=1, pl=true, pseudo=false)
    # dec_vec = (dec_v1 .+ dec_v2 .+ dec_v3) ./ 3. 

    # decay constant from ratio
    dec_ps = get_f_tm(pp_tm, m_tm_ps, ens, path_plat_tm_dps, pl=true )
    dec_v1 = get_fstar_tm(a1a1_tm, m_tm_vec1, ens, path_plat_tm_dvec, pl=true)
    dec_v2 = get_fstar_tm(a2a2_tm, m_tm_vec2, ens, path_plat_tm_dvec, pl=false)
    dec_v3 = get_fstar_tm(a3a3_tm, m_tm_vec3, ens, path_plat_tm_dvec, pl=false)
    dec_vec = (dec_v1 .+ dec_v2 .+ dec_v3) ./ 3. 

    close("all")
    #======== DECAY CONSTANT PLOTTER ===========#
    #=
    for i in 1:length(data)
        y0s          = getfield(mat_dec_ps[i], :y0)
        aux_data_ps  = data[i] .* [ sqrt(2 / m_tm_ps[i]^3) * sum(mu_list[i]) for k in 1:length(data[i]) ]
        aux_data_vec = datav1[i] .* [  za(ensinfo[k].beta) * sqrt(2 / m_tm_vec1[i])  for j in 1:length(data[i]) ] 
        uwerr.(aux_data_ps)
        uwerr.(aux_data_vec)
        dec_v1[i] *= za(ensinfo[k].beta)
        uwerr(dec_ps[i])
        uwerr(dec_v1[i])
        xx           = collect(64:length(aux_data_ps)+63)
        v , e        = value(dec_ps[i]), err(dec_ps[i])
        v_vec, e_vec = value(dec_v1[i]), err(dec_v1[i]) 
        plat_ps      =  select_plateau(ensinfo[k], mu_list, path_plat_tm_dps)[i] .- y0s
        plat_vec     =  select_plateau(ensinfo[k], mu_list, path_plat_tm_dvec)[i] .- y0s
        
        fig = figure(figsize=(10,6))
        errorbar(xx, value.(aux_data_ps), err.(aux_data_ps), fmt="s", mfc="none", label=L"$f_{D_{(s)}}$")
        errorbar(xx[1:38], value.(aux_data_vec)[1:38], err.(aux_data_vec)[1:38], fmt="s", mfc="none", label=L"$f_{D_{(s)}}^*$")
        fill_between(plat_ps[1]+62:plat_ps[2]+62, v-e, v+e, alpha=0.6 )
        fill_between(plat_vec[1]+62:plat_vec[2]+62, v_vec-e_vec, v_vec+e_vec, alpha=0.6 )
        xlabel(L"$x_0/a$")
        ylabel(L"$af$")
        xlim(65, 110)
        title("$(ensembles(dec_ps[i])[1]) " * L"\mu_1="*"$(mu_list[i][1]) " *L" \mu_2="*"$(mu_list[i][2]) " )
        ylim(v*0.95, v_vec*1.1)
        legend(fontsize=20)
        display(fig)
        t = "$(ensembles(dec_ps[i])[1])_dec_"*"mu_1=$(mu_list[i][1]) " *"_mu_2=$(mu_list[i][2]) "*".pdf"
        savefig(joinpath(path_plot,t))
        close()
    end
    =#

    # divide different mass sectors for storage
    # ps masses
    mll_tm = get_ll(mu_list, m_tm_ps, ens.deg)                     # ll
    mls_tm = ens.deg ? mll_tm : get_ls(mu_list, m_tm_ps, ens.deg)  # ls
    mlh_tm = get_lh(mu_list, m_tm_ps, ens.deg)                     # lh
    msh_tm = ens.deg ? mlh_tm : get_sh(mu_list, m_tm_ps, ens.deg)  # sh
    mhh_tm = get_hh(mu_list, m_tm_ps, ens.deg)                     # hh
    # vec masses 
    mlh_star_tm = get_lh(mu_list, m_tm_vec, ens.deg)
    msh_star_tm = ens.deg ? mlh_star_tm : get_sh(mu_list, m_tm_vec, ens.deg)
    mhh_star_tm = get_hh(mu_list, m_tm_vec, ens.deg)
    # ps decays 
    flh_tm = get_lh(mu_list, dec_ps, ens.deg)
    fsh_tm = ens.deg ? flh_tm : get_sh(mu_list, dec_ps, ens.deg)
    fhh_tm = get_hh(mu_list, dec_ps, ens.deg)
    # vec decays 
    flh_star_tm = get_lh(mu_list, dec_vec, ens.deg)
    fsh_star_tm = ens.deg ? flh_tm : get_sh(mu_list, dec_vec, ens.deg)
    fhh_star_tm = get_hh(mu_list, dec_vec, ens.deg)

    mul, mus, muh = get_mu(mu_list, ens.deg)
    t0_aux = sqrt(8 * t0_ens)
    t0vec_aux = fill(t0_aux, length(muh))
    #Dict: Masses and decay constants
    obs_tm[k] = OrderedDict(
        "mll"      => t0_aux     * mll_tm,
        "mls"      => t0_aux     * mls_tm,
        "mlh"      => t0vec_aux .* mlh_tm,
        "msh"      => t0vec_aux .* msh_tm,
        "mhh"      => t0vec_aux .* mhh_tm,
        "flh"      => t0vec_aux .* flh_tm,
        "fsh"      => t0vec_aux .* fsh_tm,
        "fhh"      => t0vec_aux .* fhh_tm,
        "mlh_star" => t0vec_aux .* mlh_star_tm,
        "msh_star" => t0vec_aux .* msh_star_tm,
        "mhh_star" => t0vec_aux .* mhh_star_tm,
        "flh_star" => t0vec_aux .* flh_star_tm,
        "fsh_star" => t0vec_aux .* fsh_star_tm,
        "fhh_star" => t0vec_aux .* fhh_star_tm,
        "muh"      => t0vec_aux .* muh
    )

   # MASS SHIFT TM OBSERVABLES
    if mass_shift 
        # read deltam from bdio
        fname = joinpath(path_bdio_md, string(ens.id,".bdio"))
        deltam = read_BDIO(fname, "md", "deltam")[1]
        # read dS/dm
        dSdm = read_pbp(path_md, ens.id)
        # definitions
        prim_obs = vcat(pp_tmW...)
        obs_tm_shifted[k] = deepcopy(obs_tm[k])
        ## LOOOP OVER OBSERVABLES + MASS SHIFT
        for (key, value) in obs_tm[k] 
            if isa(value, uwreal) # if 1 uwreal
                mds1, mds2 = md_sea(value, dSdm, prim_obs, W) .+ md_sea(value, dSdm, YW, W)
                obs_tm_shifted[k][key] = obs_tm[k][key] + deltam * (2 * mds1 + mds2)
            elseif isa(value, Vector{uwreal}) # if vector uwreal
                for (i, v) in enumerate(value)
                    mds1, mds2 = md_sea(v, dSdm, prim_obs, W) .+ md_sea(v, dSdm, YW, W)
                    obs_tm_shifted[k][key][i] = obs_tm[k][key][i] + deltam * (2 * mds1 + mds2)
                end
            end
        end
    end
end

##
#====================== WRITE BDIO =====================#

if mass_shift
    #TM SHIFTED
    for (k, obs) in enumerate(obs_tm_shifted) #loop over ensembles
        ens = ensinfo[k].id
        p = joinpath(path_bdio_tm_shifted, string(ens, ".bdio"))
        fb = BDIO_open(p, "w", ens) 
        uinfo = 0
        for (key, value) in obs #loop over observables
            if isa(value, uwreal)
                write_uwreal(value, fb, uinfo)
            elseif isa(value, Vector{uwreal})
                [write_uwreal(v, fb, uinfo) for v in value]
            end
            uinfo += 1
        end
        BDIO_close!(fb)
    end
end
# TM
for (k, obs) in enumerate(obs_tm) #loop over ensembles
    ens = ensinfo[k].id
    p = joinpath(path_bdio_tm, string(ens, ".bdio"))
    fb = BDIO_open(p, "w", ens)
    uinfo = 0
    for (key, value) in obs #loop over observables
        if isa(value, uwreal)
            write_uwreal(value, fb, uinfo)
        elseif isa(value, Vector{uwreal})
            [write_uwreal(v, fb, uinfo) for v in value]
        end
        uinfo += 1
    end
    BDIO_close!(fb)
end

