#################################
# This file was created by Alessandro Conigli - 2022
# 
# Perform chiral- continuum extrapolations and AIC analysis
#  
#
#
# 
#  
##################################
using Revise, ADerrors, BDIO, juobs, PyPlot, LaTeXStrings, DelimitedFiles, Combinatorics
using LsqFit 

include("gevp_types.jl")
include("gevp_reader.jl")
include("gevp_tools.jl")
include("const.jl")
include("gevp_func_comb.jl")
include("gevp_plotter.jl")

# redefine + * operations
import Base: +,*,/
#uwreal +
function +(uw::uwreal, vec::Vector{uwreal})
    n = size(vec,1)
    return fill(uw, n) .+ vec
end
function +(vec::Vector{uwreal}, uw::uwreal )  uw+vec  end
#uwreal *
function *(uw::uwreal, vec::Vector{uwreal})
    n = size(vec,1)
    return fill(uw, n) .* vec
end
function *(vec::Vector{uwreal}, uw::uwreal )  uw*vec  end
#any +
function +(uw::uwreal, vec::Vector{Any})
    n = size(vec,1)
    return fill(uw, n) .+ vec
end
function +(vec::Vector{Any}, uw::uwreal )  uw+vec  end
#any *
function *(uw::uwreal, vec::Vector{Any})
    n = size(vec,1)
    return fill(uw, n) .* vec
end
function *(vec::Vector{Any}, uw::uwreal )  uw*vec  end
#float +
function +(uw::uwreal, vec::Vector{Float64})
    n = size(vec,1)
    return fill(uw, n) .+ vec
end
function +(vec::Vector{Float64}, uw::uwreal )  uw+vec  end
#float *
function *(uw::uwreal, vec::Vector{Float64})
    n = size(vec,1)
    return fill(uw, n) .* vec
end
function *(vec::Vector{Float64}, uw::uwreal )  uw*vec  end


#============= INPUT PAR ===========#

# desktop parameters
#include("/home/alessandro/juobs/constants/juobs_const.jl")
#path_data ="/home/alessandro/Desktop/data/heavy_3pt/3pt"
#path_rw = "/media/alessandro/4277fef2-edc5-4e0d-89cb-f5d1d44fbc8c/data/rwf"
#path_plot = "/home/alessandro/google-drive/phd/secondment/3pf test/analysis/plots"

# macbook 15 parameters
#include("/Users/ale/juobs/constants/juobs_const.jl")
#path_plot = "/Users/ale/Il mio Drive/phd/analysis/charm_gevp_2022/plots"
#path_results = "/Users/ale/Il mio Drive/phd/analysis/charm_gevp_2022/bdio"
#path_t0_and_dm = "/Users/ale/Desktop/data"

include("/Users/alessandroconigli/Lattice/juobs/constants/juobs_const.jl")
path_plot = "/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/plots"
path_results = "/Users/alessandroconigli/MyDrive/phd/analysis/charm_gevp_2022/bdio"
path_t0_and_dm = "/Users/alessandroconigli/Lattice/data"


# latex-style labels 
PyPlot.rc("font", family="sans serif", size=12)
rcParams = PyPlot.PyDict(PyPlot.matplotlib."rcParams")
rcParams["text.usetex"] = false
rcParams["mathtext.fontset"]  = "cm"
#rcParams["font.size"] =11
rcParams["axes.labelsize"] =22



const ens_list = ["H101", "H102r002", "H400", "N202", "N203", "N200", "J303", "N300"]
const sector = Dict("pseudo"=>true, "vector"=>true)

# loading ensemble information
ensinfo = Vector{EnsInfo}(undef, length(ens_list))
for i in 1:length(ens_list)
    ens = ens_list[i]
    try
        ensinfo[i] = EnsInfo(ens, ens_db[ens] )
    catch
        error("The ensemble id ", ens, " was not found in the const.jl ens_db database. Please check the ensemble id or update the database")
    end
end

##########################
## initialize containers
##########################

#charm quark mass
muc_t0       = Vector{Vector{uwreal}}(undef, length(ensinfo))
# pseudoscalar masses 
if sector["pseudo"]
    mps_ll_t0    = Vector{uwreal}(undef, length(ensinfo)) # pion mass 
    mps_ls_t0    = Vector{uwreal}(undef, length(ensinfo)) # kaon mass
    mps_ss_t0    = Vector{uwreal}(undef, length(ensinfo)) # ss mass
    mps_lh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # D mass
    mps_sh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # Ds mass
    mps_hh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # η_c mass
    # pseudoscalar decays
    fps_ll_t0    = Vector{uwreal}(undef, length(ensinfo)) # pion decay 
    fps_ls_t0    = Vector{uwreal}(undef, length(ensinfo)) # kaon decay
    fps_ss_t0    = Vector{uwreal}(undef, length(ensinfo)) # ss decay
    fps_lh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # D decay
    fps_sh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # Ds decay
    fps_hh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # η_c decay
end

if sector["vector"]
    # vector masses 
    mvec_ll_t0    = Vector{uwreal}(undef, length(ensinfo)) # rho mass 
    mvec_ls_t0    = Vector{uwreal}(undef, length(ensinfo)) # ls vec mass
    mvec_ss_t0    = Vector{uwreal}(undef, length(ensinfo)) # ss vec mass
    mvec_lh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # D* mass
    mvec_sh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # Ds* mass
    mvec_hh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # J/Ψ mass
    # vector decays
    fvec_ll_t0    = Vector{uwreal}(undef, length(ensinfo)) # rho decay 
    fvec_ls_t0    = Vector{uwreal}(undef, length(ensinfo)) # ls vec decay
    fvec_ss_t0    = Vector{uwreal}(undef, length(ensinfo)) # ss vec decay
    fvec_lh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # D* decay
    fvec_sh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # Ds* decay
    fvec_hh_t0    = Vector{Vector{uwreal}}(undef, length(ensinfo)) # J/Ψ decay    
end

####################### 
## read bdio files 
#######################

# read t0_shifted from BDIO
#dm = read_BDIO(path_t0_and_dm, "delta")
t0_shifted =  read_BDIO(path_t0_and_dm, "t0")
uwerr.(t0_shifted)
label = vcat(ensembles.(t0_shifted)...)
#
N_mh = fill(3, length(ensinfo))

use_t0_from_bdio = true

for i in 1:length(ensinfo)
    ens = ensinfo[i]
    idx_t0 = findall(x-> x==ens.id, label)[1]
    t0_val = use_t0_from_bdio ? t0_shifted[idx_t0] : t0(ens.beta) 
    println(t0_val)
    println("Reading observables for ensemble $(ens.id)" )
    aux_path   = joinpath(path_results, "charm_mass")
    muc_t0[i]  = muh_val[ens.id ] .* [sqrt(8 * t0_val)  / ZP(ens.beta)    for k in 1:N_mh[i]]   #read_BDIO(aux_path, ens.id)
    # implement t0 from bdio and not only from database
    if sector["pseudo"]
        # read ps masses and decays
        aux_m            = read_BDIO(joinpath(path_results, "ps_masses"), ens.id)
        aux_f            = read_BDIO(joinpath(path_results, "ps_decays"), ens.id)
        mps_ll_t0[i]     = sqrt(8 * t0_val) * aux_m[1]  
        fps_ll_t0[i]     = sqrt(8 * t0_val) * aux_f[1]  
        if ens.deg
            mps_ss_t0[i] = mps_ls_t0[i] = mps_ll_t0[i]
            mps_lh_t0[i] = mps_sh_t0[i] = aux_m[2:4] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]  
            mps_hh_t0[i] =                aux_m[5:7] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]             

            fps_ss_t0[i] = fps_ls_t0[i] = fps_ll_t0[i]
            fps_lh_t0[i] = fps_sh_t0[i] = aux_f[2:4] .* [sqrt(8*t0_val) for k in 1:N_mh[i]] 
            fps_hh_t0[i] =                aux_f[5:7] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]                
        else
            mps_ls_t0[i] = aux_m[2]  * sqrt(8*t0_val)   
            mps_ss_t0[i] = aux_m[3]  * sqrt(8*t0_val)     
            mps_lh_t0[i] = aux_m[4:6]   .* [sqrt(8*t0_val) for k in 1:N_mh[i]]   
            mps_sh_t0[i] = aux_m[7:9]   .* [sqrt(8*t0_val) for k in 1:N_mh[i]]   
            mps_hh_t0[i] = aux_m[10:12] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]   

            fps_ls_t0[i] = aux_f[2] * sqrt(8*t0_val) 
            fps_ss_t0[i] = aux_f[3] * sqrt(8*t0_val) 
            fps_lh_t0[i] = aux_f[4:6]   .* [sqrt(8*t0_val) for k in 1:N_mh[i]]  
            fps_sh_t0[i] = aux_f[7:9]   .* [sqrt(8*t0_val) for k in 1:N_mh[i]]  
            fps_hh_t0[i] = aux_f[10:12] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]  
        end
    end
    
    if sector["vector"]
        # read ps masses and decays
        aux_m             = read_BDIO(joinpath(path_results, "vec_masses"), ens.id)
        aux_f             = read_BDIO(joinpath(path_results, "vec_decays"), ens.id)
        mvec_ll_t0[i]     = sqrt(8*t0_val)  * aux_m[1] 
        fvec_ll_t0[i]     = sqrt(8*t0_val) * aux_f[1] 
        if ens.deg
            mvec_ss_t0[i] = mvec_ls_t0[i] = mvec_ll_t0[i]
            mvec_lh_t0[i] = mvec_sh_t0[i] = aux_m[2:4] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
            mvec_hh_t0[i] = aux_m[5:7]                 .* [sqrt(8*t0_val) for k in 1:N_mh[i]]

            fvec_ss_t0[i] = fvec_ls_t0[i] = fvec_ll_t0[i]
            fvec_lh_t0[i] = fvec_sh_t0[i] = aux_f[2:4] .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
            fvec_hh_t0[i] = aux_f[5:7]                 .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
        else
            mvec_ls_t0[i] = aux_m[2]                    * sqrt(8*t0_val) 
            mvec_ss_t0[i] = aux_m[3]                    * sqrt(8*t0_val)
            mvec_lh_t0[i] = aux_m[4:6]                 .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
            mvec_sh_t0[i] = aux_m[7:9]                 .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
            mvec_hh_t0[i] = aux_m[10:12]               .* [sqrt(8*t0_val) for k in 1:N_mh[i]]

            fvec_ls_t0[i] = aux_f[2]                    * sqrt(8*t0_val)
            fvec_ss_t0[i] = aux_f[3]                    * sqrt(8*t0_val)
            fvec_lh_t0[i] = aux_f[4:6]                 .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
            fvec_sh_t0[i] = aux_f[7:9]                 .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
            fvec_hh_t0[i] = aux_f[10:12]               .* [sqrt(8*t0_val) for k in 1:N_mh[i]]
        end
    end
    

end

################################################################
## Collect physical values quantities for matching and fitting
################################################################



sqrt8t0      = Vector{uwreal}(undef, 0)
#fb = BDIO_open("/Users/ale/Desktop/data/t0_shifted.bdio")
for i in 1:length(ensinfo)
    ens = ensinfo[i]
    idx_t0 = findall(x-> x==ens.id, label)[1]
    t0_val = use_t0_from_bdio ? t0_shifted[idx_t0] : t0(ens.beta) 
    [push!(sqrt8t0, sqrt(8 *t0_val)) for k in 1:N_mh[i]]
end

# lattice units 
phih_eta     = vcat(mps_hh_t0...)
phih_flav    = 2/3 .* vcat(mps_lh_t0...) + 1/3 .* vcat(mps_sh_t0...)
phih_spav    = 1/4 .* (phih_flav .+ 2 .* vcat(mvec_lh_t0...) .+ vcat(mvec_sh_t0...))
phi2         = vcat([fill(mps_ll_t0[i]^2, N_mh[i]) for i in 1:length(ensinfo)]...) #for i in 1:length(ensinfo)] 
phi4         = vcat([fill(1/2 * mps_ll_t0[i]^2 +  mps_ls_t0[i]^2, N_mh[i]) for i in 1:length(ensinfo)]...)# 1/2 .* mps_ll_t0.^2 .+  mps_ls_t0 .^2
# physical units
phih_eta_ph  = M_values[5] * t0_ph[1] / hc
phih_flav_ph = (2/3 * M_values[1] + 1/3 * M_values[3]) * t0_ph[1] / hc
phih_spav_ph = 1/4 * (2/3 * M_values[1] + 1/3 * M_values[3] + 2* M_values[2] + M_values[4]) * t0_ph[1] / hc
phi2_ph      = (t0_ph[1] * 134.8 / hc)^2  # try 139 or 134.8 isospin breaking pion mass 
phi4_ph      = uwreal([1.098, 0.0010], "phi4_phys")
# xdata array to feed the fit routine for different categories 

xdata_flav = [ 1 ./ sqrt8t0.^2 phi2 phih_flav]
xdata_eta  = [ 1 ./ sqrt8t0.^2 phi2 phih_eta] 
xdata_spav = [ 1 ./ sqrt8t0.^2 phi2 phih_spav] 



########################
## Creating categories 
########################


# WPM WINDOWS
wpmm = Dict{String, Vector{Float64}}()
wpmm["N202"] = [-1.0, 2.0,-1.0, -1.0]
#wpmm["N200"] = [-1.0, 2.0,-1.0, -1.0]
wpmm["J303"] = [-1.0, 2.0,-1.0, -1.0]

#wpmm[300] = [-1.0, 1.5,-1.0, -1.0]


cat_muc_eta  = Cat(vcat(muc_t0...), xdata_eta,  phih_eta_ph,  "muc_etac")
cat_muc_flav = Cat(vcat(muc_t0...), xdata_flav, phih_flav_ph, "muc_flav")
#cat_muc_spav = Cat(vcat(muc_t0...), xdata_spav, phih_spav_ph, "muc_spav_matching")

cat_muc_tot = [cat_muc_eta, cat_muc_flav]

# fit all models for categories
fit_over_cat!(cat_muc_tot, model_charm, phi2_ph, n_param_j, weight_dof=2 )

muc_av_eta = aic_model_ave(cat_muc_eta)
muc_av_flav = aic_model_ave(cat_muc_flav)

# working plotter
idx_toplot = 1
plot_cat(cat_muc_tot[idx_toplot], L"$M_c^{\mathrm{RGI}}(N_f=3)\mathrm{MeV}$" , save=false)
plot_chi_charm(cat_muc_tot[idx_toplot], vcat(model_charm_aux...),  L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, n=1)
plot_cl_charm(cat_muc_tot[idx_toplot], vcat(model_charm_aux...), L"$ciao$",phi2_ph, ensinfo, mrat=Mrat, n=1 )
plot_all_cl_charm(cat_muc_tot[1:2], vcat(model_charm_aux...),L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, mrat=Mrat )# , p=path_plot)
plot_chi2_vs_exp(cat_muc_flav, label_cutoff)

##
model_j = vcat(f...)
for cat in cat_muc_tot[1:2]
    for i in 1:length(model_charm)
        #try
            par, chi2_over_chiexp = fit_routine(model_j[i], value.(cat.x_tofit), cat.obs, n_param_charm[i],  wpm=wpmm)
            aux =  Mrat * (par[1] + par[2]*phi2_ph + par[3]*cat.phih_ph)
            uwerr(aux)
            println("charm mass = ", aux)
            push!(cat.fit_res, aux)
            push!(cat.fit_param, par)
            push!(cat.chi2_corr, chi2_over_chiexp)
            push!(cat.n_param, n_param_charm[i])
            push!(cat.dof,  length(cat.obs)-n_param_charm[i] ) 
            push!(cat.aic, cat.chi2_corr[i]*cat.dof[i] +5*n_param_charm[i]) 
            println("aic = ", cat.aic[i])
            println("\n")
        #catch
        #    println("something went wrong")
        #end
    end
end

muc_av_eta = aic_model_ave(cat_muc_eta)
muc_av_flav = aic_model_ave(cat_muc_flav)
muc_av_spav = aic_model_ave(cat_muc_spav)



plot_cat(cat_muc_eta, L"$M_c^{\mathrm{RGI}}(N_f=3)\mathrm{MeV}$" , save=false)
plot_cl_charm(cat_muc_eta, model_charm, L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo)
plot_all_cl_charm(cat_muc_tot[1:2], model_charm,L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, n=[0,1,12] )# , p=path_plot)
plot_chi_charm(cat_muc_flav, model_charm, L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo)
plot_hist(cat_muc_tot[1:2]  )

## final results
mc_fin, mc_syst = category_av([cat_muc_eta, cat_muc_flav]) 
mc_fin_ph = mc_fin * hc /t0_ph[1]
mc_syst_ph = mc_syst * hc /t0_ph[1]
uwerr(mc_fin_ph)
uwerr(mc_syst_ph)
println("Final result for RGI charm quark mass in physical units: ", mc_fin_ph)
println("Systematic error coming from model average: ", mc_syst_ph)

######
## test muc with javier functions 
for cat in cat_muc_tot[1:1]
    println("\n", cat.info, "\n")
    for j  in 1:length(f)             # looping over number of parameters
        for k in 1:length(f[j]) # looping over combinations with fixed parameters 
            println(" j is :", j, " k is equal: ", k)
            #try
                par, chi_corr = fit_routine(f[j][k], value.(cat.x_tofit), cat.obs, n_param2[j], wpm=wpmm)
                aux = par[1] + par[2]*phi2_ph + par[3]*(cat.phih_ph)
                println(aux)
                uwerr(aux)
                push!(cat.fit_res, aux)
                push!(cat.fit_param, par)
                push!(cat.chi2_corr, chi_corr)
                push!(cat.n_param, n_param2[j])
                push!(cat.aic, chi_corr +2*n_param2[j])
            #catch
            #    println("There was a problem... Probably the analysis failed for some error id.")
            #    println("The result has been excluded from the category")
            #end    
        end
    end
end


#########################
## fits for MD and MDS
#########################

cat_mD_eta   = Cat(vcat(mps_lh_t0...), xdata_eta, phih_eta_ph, "mD_etac_matching")
cat_mD_flav  = Cat(vcat(mps_lh_t0...), xdata_flav, phih_flav_ph, "mD_flav_matching")

cat_mDs_eta  = Cat(vcat(mps_sh_t0...), xdata_eta, phih_eta_ph, "mDs_etac_matching")
cat_mDs_flav = Cat(vcat(mps_sh_t0...), xdata_flav, phih_flav_ph, "mDs_flav_matching")

cat_D_Ds_masses =[cat_mD_eta, cat_mD_flav, cat_mDs_eta, cat_mDs_flav]
for cat in cat_D_Ds_masses[1:1]
    println(cat.info)
    for i in 1:length(model_charm)
        #try
            par, chi2_over_chi2exp = fit_routine(model_charm[i], value.(cat.x_tofit), cat.obs, n_param_charm[i], wpm=wpmm)
            aux = par[1] + par[2]*phi2_ph + par[3]*(cat.phih_ph)
            uwerr(aux)
            println("mD mass  = ", aux)
            push!(cat.fit_res, aux)
            push!(cat.fit_param, par)
            push!(cat.chi2_corr, chi2_over_chi2exp)
            push!(cat.n_param, n_param_charm[i])
            push!(cat.dof,  length(cat.obs)-n_param_charm[i] ) 
            push!(cat.aic, cat.chi2_corr[i]*cat.dof[i] +2*n_param_charm[i])    

            println("chi2 / chiexp = ", chi2_over_chi2exp)
            println("\n")
        #catch
        #    println("something went wrong")
        #end
    end
end

mD_av = aic_model_ave(cat_mD_flav)


## plots 
plot_cont_lim_phi2_meson_mass(cat_mDs_eta, model_charm, L"$\sqrt{8t_0}M_{D_s}$", dec=false, save=false)
plot_chi_masses(cat_mDs_flav, model_charm, L"$\sqrt{8t_0}M_{D_s}$", phi2_ph, ensinfo, n=1 , p=path_plot)

plot_cont_lim(cat_muc_eta, model_charm, L"$\sqrt{8t_0}M_{D_s}$", save=false)
plot_cat(cat_muc_flav, L"$\sqrt{8t_0}M_{D}$" )

################
## fit for fD(s)
################
xdata_eta_dec = deepcopy(xdata_eta)
xdata_eta_dec[:,3] = 1.0 ./sqrt.(xdata_eta[:,3])
xdata_flav_dec = deepcopy(xdata_flav)
xdata_flav_dec[:,3] = 1.0 ./sqrt.(xdata_flav[:,3])

cat_fD_eta  = Cat(vcat(fps_lh_t0...), xdata_eta, phih_eta_ph, "fD_etac_matching")
cat_fD_flav = Cat(vcat(fps_lh_t0...), xdata_flav, phih_flav_ph, "fD_flav_matching")

cat_fDs_eta  = Cat(vcat(fps_sh_t0...), xdata_eta, phih_eta_ph, "fDs_etac_matching")
cat_fDs_flav = Cat(vcat(fps_sh_t0...), xdata_flav, phih_flav_ph, "fDs_flav_matching")

cat_fD_fDs = [cat_fD_eta, cat_fD_flav, cat_fDs_eta, cat_fDs_flav]
for cat in cat_fD_fDs[1:1]
    println(cat.info)
    for i in 1:length(model_decay)
        #try
            par, chi2_over_chi2exp = fit_routine(model_decay[i], value.(cat.x_tofit), cat.obs, n_param_decay[i],   wpm=wpmm)
            aux = par[1] + par[2]*phi2_ph + par[3] * cat.phih_ph  # /sqrt((cat.phih_ph))
            uwerr(aux)
            println("fD(s) decays  = ", aux)
            push!(cat.fit_res, aux)
            push!(cat.fit_param, par)
            push!(cat.chi2_corr, chi2_over_chi2exp)
            push!(cat.n_param, n_param_decay[i])
            push!(cat.dof,  length(cat.obs)-n_param_decay[i] ) 
            push!(cat.aic, cat.chi2_corr[i]*cat.dof[i] +2*n_param_decay[i]) 
            println("chi2 / chiexp = ", chi2_over_chi2exp)
            println("aic = ", cat.aic[i])
            println("\n")
        #catch
        #    println("something went wrong")
        #end
    end
end

fD_av_eta  = aic_model_ave(cat_fD_eta) 
fD_av_flav = aic_model_ave(cat_fD_flav)
fDs_av_eta  = aic_model_ave(cat_fDs_eta) 
fDs_av_flav = aic_model_ave(cat_fDs_flav)

plot_cont_lim_phi2(cat_fD_eta, model_decays, L"$\sqrt{8t_0}f_{D_s}$", dec=true, save=false)
plot_cat(cat_fD_eta, L"ciao")

plot_cl_charm(cat_fD_eta, model_decay, L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, n=1)
plot_chi_charm(cat_fD_eta, model_decay, L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, n=2)






## testing fit charm with Javier function definition

cat_fD_eta  = Cat(vcat(fps_lh_t0...), xdata_eta, phih_eta_ph, "fD_etac_matching")
cat_fD_flav = Cat(vcat(fps_lh_t0...), xdata_flav, phih_flav_ph, "fD_flav_matching")

cat_fDs_eta  = Cat(vcat(fps_sh_t0...), xdata_eta, phih_eta_ph, "fDs_etac_matching")
cat_fDs_flav = Cat(vcat(fps_sh_t0...), xdata_flav, phih_flav_ph, "fDs_flav_matching")

cat_fD_fDs = [cat_fD_eta, cat_fD_flav, cat_fDs_eta, cat_fDs_flav]

for cat in cat_fD_fDs[1:1]
    println("\n", cat.info, "\n")
    for j  in 1:length(f)             # looping over number of parameters
        for k in 1:length(f[j]) # looping over combinations with fixed parameters 
            println(" j is :", j, " k is equal: ", k)
            #try
                par, chi_corr = fit_routine(f[j][k], value.(cat.x_tofit), cat.obs, n_param2[j], wpm=wpmm)
                aux = par[1] + par[2]*phi2_ph + par[3] / sqrt(cat.phih_ph)
                println(aux)
                uwerr(aux)
                push!(cat.fit_res, aux)
                push!(cat.fit_param, par)
                push!(cat.chi2_corr, chi_corr)
                push!(cat.n_param, n_param2[j])
                push!(cat.aic, chi_corr +2*n_param2[j])
            #catch
            #    println("There was a problem... Probably the analysis failed for some error id.")
            #    println("The result has been excluded from the category")
            #end    
        end
    end
end



fD_av_eta  = aic_model_ave(cat_fD_eta) 
fD_av_flav = aic_model_ave(cat_fD_flav)
fDs_av_eta  = aic_model_ave(cat_fDs_eta) 
fDs_av_flav = aic_model_ave(cat_fDs_flav)

plot_cont_lim_phi2(cat_fD_eta, model_decays, L"$\sqrt{8t_0}f_{D_s}$", dec=true, save=false)
plot_cat(cat_fD_eta, L"ciao")

plot_cl_charm(cat_fD_eta, model_decay, L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, n=1)
plot_chi_charm(cat_fD_eta, model_decay, L"$\sqrt{8t_0}Z_M^{tm}\mu_c$", phi2_ph, ensinfo, n=2)



## test fit fD(s) with javier function

par_eta, chi  = fit_routine(f[2][1], xdata_eta, vcat(fps_lh_t0...), 6 , covar=true)
par_flav, chi = fit_routine(f[1][1], xdata_flav, vcat(fps_lh_t0...), 5 , covar=true, wpm=wpmm)

par_eta2, chi  = fit_routine(f[2][1], xdata_eta, vcat(fps_sh_t0...), 6 , covar=true)
par_flav, chi = fit_routine(f[1][1], xdata_flav, vcat(fps_lh_t0...), 5 , covar=true, wpm=wpmm)

test_fd_eta  = par_eta[1] + par_eta[2]*phi2_ph + par_eta[3]*phih_eta_ph
test_fd_flav = par_flav[1] + par_flav[2]*phi2_ph + par_flav[3]*phih_flav_ph

uwerr(val_ph_eta)
uwerr(val_ph_flav)


##
muh_target_arr = []
fig = figure()
    count = 1
    for i in 1:length(ensinfo)
        xx = 1 / (8*t0(ensinfo[i].beta))
        muh_target_eta  = basemodel([xx unique(phi2)[i] phih_eta_ph], value.(par_eta))[1]
        muh_target_flav = basemodel([xx unique(phi2)[i] phih_flav_ph], value.(par_flav))[1]

        #muh_target = match_muc(value.(muc_t0[i]), mps_lh_t0[i], mps_sh_t0[i], phih_flav_ph)
        uwerr(muh_target_eta)
        uwerr(muh_target_flav)

        errorbar(value(xx), value(muh_target_eta), err(muh_target_eta), fmt="s"  , capsize=2, mfc="none", color="red", ecolor="red")#, label=L"$m_{η_c}$" )
        errorbar(value(xx)+0.0005, value(muh_target_flav), err(muh_target_flav), fmt="s", capsize=2, mfc="none", color="royalblue", ecolor="royalblue")#,label=L"$m_{\bar D}$")

        
    end   
    
    xarr = Float64.(range(0.0, stop=0.4, length=100))
    y_eta = Vector{uwreal}(undef, 100)
    y_flav = Vector{uwreal}(undef, 100)
    
    for i in 1:100
        y_eta[i] = par_eta[1] + par_eta[2]*phi2_ph + par_eta[3]*phih_eta_ph + par_eta[4]*xarr[i]  
        y_flav[i]= par_flav[1] + par_flav[2]*phi2_ph + par_flav[3]*phih_flav_ph + par_flav[4]*xarr[i] 

    end
    uwerr.(y_eta)
    uwerr.(y_flav)

    fill_between(xarr, value.(y_eta) .+ err.(y_eta),value.(y_eta) .- err.(y_eta) , alpha=0.2,zorder=1, lw=0.8, color="red")
    fill_between(xarr, value.(y_flav) .+ err.(y_flav),value.(y_flav) .- err.(y_flav) , alpha=0.2,zorder=1, lw=0.8, color="royalblue")

    errorbar(0, value(val_ph_eta), err(val_ph_eta), fmt="s", ms=8, capsize=2, color="red",ecolor="red")
    errorbar(0.0005, value(val_ph_flav), err(val_ph_flav), fmt="s", capsize=2, ms=8, color="royalblue",ecolor="royalblue")

    xlim(-0.001, 0.055)
    ylim(2.9, 3.3)
    xlabel(L"$a^2/8t_0$")
    ylabel(L"$\sqrt{8t_0}Z_M^{tm}\mu_c$")
    legend(L"$m_{η_c}$", L"$m_{\bar D}$")
display(fig)
