@doc raw"""
    read_data(ens::String, g1::String="G5", g2::String="G5")
Given the ensemble id and the Dirac structure, this method reads the data in path_data and
returns a CData object. Replicas are automatically detected and loaded if present.
"""
function read_data(ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    path = joinpath(path_data, ens)
    rep = filter(x->occursin("mesons.dat", x), readdir(path, join=true))
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons(rep[1], g1, g2, id=ens, legacy=legacy)) : (return read_mesons(rep, g1, g2, id=ens, legacy=legacy))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", path)
    end
end
function read_rw(ens::String)
    rep = filter(x->occursin(ens, x), readdir(path_rw, join=true))
    if length(rep)!=0
        try
            length(rep) == 1 ? (return read_ms1(rep[1])) : (return read_ms1.(rep)) 
        catch
            length(rep) == 1 && length(rep)!=0 ? (return read_ms1(rep[1], v="1.4")) : (return read_ms1.(rep, v="1.4")) 
        end
    else
        error("ms1.dat file not found for ensemble ", ens, " in path ", path_rw)
    end
end
function read_mass_dev(ens::String)
    rep = filter(x->occursin(ens,x), readdir(path_md, join=true))
    try
        if length(rep) == 1
            return [read_md(rep[1])]
        else
            return [read_md(rep[1]), read_md(rep[2])]
        end
    catch
        error("pbp.dat file not found for ensemble ", ens, " in path ", path_md)
    end
end
function read_t0(ens::String)
    path = joinpath(path_data, ens)
    rep = filter(x->occursin("ms.dat", x), readdir(path, join=true))
    if length(rep)!=0
        length(rep) == 1 ? (return read_ms(rep[1])) : (return read_ms.(rep)) 
    else
        error("ms.dat file not found for ensemble ", ens, " in path ", path)
    end
end
function get_corr(ens::EnsInfo, g1::String="G5", g2::String="G5"; rw::Bool=false, legacy::Bool=false)
    aux = read_data(ens.id, g1, g2, legacy=legacy)
    if ens.id =="J303"
        truncate_data!(aux, 739)
    end
    if ens.id =="H101"
        #truncate_data!(aux, [878, 999])
    end
    !rw ? obs = corr_obs.(aux, L=ens.L) : obs = corr_obs.(aux, L=ens.L, rw=read_rw(ens.id))
    return obs
end
function get_t0(ens::EnsInfo; rw::Bool=false, pl::Bool=false)
    data = read_t0(ens.id)
    plat = select_plateau(ens) 
    !rw ? t0 = comp_t0(data, plat, L=ens.L, pl=pl) : t0 = comp_t0(data, plat, L=ens.L, pl=pl, rw=read_rw(ens.id))
    return t0
end
function get_mu(mu_list::Vector{Vector{Float64}}, deg::Bool)
    mu_sorted = unique(sort(minimum.(mu_list)))

    mul = mu_sorted[1]
    deg ? mus = 0.0 : mus = mu_sorted[2]
    muh = unique(maximum.(mu_list))
    muh = filter(x-> x > mul && x > mus, muh)

    return mul, mus, muh
end
function get_ll(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] == mu[2] #l-l
            return obs[k]
        end
    end
end
function get_ls(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mus in mu  #l-l
            return obs[k]
        end
    end
end
function get_lh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_lh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu) #l-h
            push!(obs_lh, obs[k])
        end
    end
    return obs_lh
end
function get_ss(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] == mu[2] #s-s
            return obs[k]
        end
    end
end
function get_sh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_sh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] != mu[2] && !(mul in mu)#s-h
            push!(obs_sh, obs[k])
        end
    end
    return obs_sh
end
function get_hh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_hh = Vector{uwreal}(undef,0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        for i =1:length(muh)
            if muh[i] in mu && mu[1] == mu[2]#h-h
                push!(obs_hh, obs[k])
            end
        end
    end
    return obs_hh
end
function comp_mat(ensinfo::EnsInfo, tot_obs::Vector{juobs.Corr}, tau::Int64)
    mat_info = Vector{MatInfo}(undef, length(tot_obs))
    for i = 1:length(tot_obs)
        a11 = tot_obs[i].obs[1:end-1-2*tau]
        a12 = tot_obs[i].obs[1+tau:end-1-tau]
        a22 = tot_obs[i].obs[1+2*tau:end-1]
        aux = juobs.get_matrix([a11, a22], [a12])
        mat_info[i] = MatInfo(ensinfo, aux, tot_obs[i].mu, tot_obs[i].y0)
    end
    return mat_info
end
function comp_mat_multigamma(ensinfo::EnsInfo,tot11::Vector{juobs.Corr}, tot12::Vector{juobs.Corr}, tot22::Vector{juobs.Corr})
    mu = getfield.(tot11,:mu)
    y0 = getfield.(tot11,:y0) .+1

    mat_info = Vector{MatInfo}(undef, length(mu))
    for i = 1:length(mu)
        a11 = tot11[i].obs[y0[i]:end-1]
        a12 = tot12[i].obs[y0[i]:end-1]
        a22 = tot22[i].obs[y0[i]:end-1]
        #res[i] = infoMatt(a11, a12, a22)
        aux = juobs.get_matrix([a11, a22], [a12])
        mat_info[i] = MatInfo(ensinfo, aux, mu[i], y0[i])
    end
    return mat_info
end
function comp_mat_lukas(ensinfo::EnsInfo, tot_obs::Vector{juobs.Corr}, dim::Int64)
    mat_info = Vector{MatInfo}(undef, length(tot_obs))
    for i in 1:length(tot_obs)
        a11 = tot_obs[i].obs[1+1:end-6]
        a22 = tot_obs[i].obs[3+1:end-4]
        a33 = tot_obs[i].obs[5+1:end-2]
        a12 = tot_obs[i].obs[2+1:end-5]
        a13 = tot_obs[i].obs[3+1:end-4]
        a23 = tot_obs[i].obs[4+1:end-3]
        aux = juobs.get_matrix([a11, a22, a33], [a12, a13, a23])
        mat_info[i] = MatInfo(ensinfo, aux, tot_obs[i].mu, tot_obs[i].y0)        
    end
    return mat_info
end
function gevp_mass(mat_obs::Vector{MatInfo}; t0::Int64=2,pl::Bool=true, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing)
    y0s = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    plat_en0 = Vector{uwreal}(undef, length(mat_obs))
    plat_en1 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list)[y0s[i]+1:end-1] #matrices to diagonalise for a given sector [i]
        evals = getall_eigvals(mat, t0)
        en = energies(evals, wpm=wpm) #ground and excited state energy
        plat = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat_meff)[i] .- y0s[i] 
        println(plat)
        plat_en0[i] = plat_av(en[1], plat)
        if pl 
            uwerr(plat_en0[i])
            xx = collect(1:length(en[1]))
            v = value(plat_en0[i])
            e = err(plat_en0[i])
            errorbar(xx, value.(en[1]), err.(en[1]), fmt="s")
            fill_between(plat[1]:plat[2], v-e, v+e )
            ylim(v*0.95, v*1.05)
            display(gcf())
            close()
        end
    end
    return plat_en0
end
# function used in lattice conf 2021 with fits to the effective energies
function gevp_mass_old(mat_obs::Vector{MatInfo}; t0::Int64=2, delta_t_in::Array{Int64}=[2,5], wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing)
    y0s = getfield.(mat_obs, :y0) 
    mu_list = getfield.(mat_obs, :mu) 
    plat_en0 = Vector{uwreal}(undef, length(mat_obs))
    plat_en1 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list)[y0s[i]+1:end-1] #matrices to diagonalise for a given sector [i]
        evals = getall_eigvals(mat, t0)
        en = energies(evals, wpm=wpm) #ground and excited state energy
        aux_mass = Vector{uwreal}(undef,0)
        for t in delta_t_in[1]:delta_t_in[2]
            try
                temp = fit_mass(en[1],t_in=t,  wpm=wpm)
                push!(aux_mass, temp)
            catch
                plat = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat_meff)[end-2] .- y0s[i] 
                println(plat)
                temp = plat_av(en[1], plat)
                uwerr(temp)
                push!(aux_mass, temp)
                println("\n Error analysis failed for ensemble ", mat_obs[i].ensinfo.id, " in the mu sector ", mat_obs[i].mu, " with time fit values :", t)
                println("\n The mass has been computed with a plateau average. Plateaux has been taken from select plateau file.\n")
            end
        end
        plat_en0[i] = param_av(aux_mass)
    end
    return plat_en0
end
function fit_mass(en_i::Array{uwreal}; t_in::Int64=3, t_fin::Int64=length(en_i)-1, wpm::Union{Nothing, Dict{String,Vector{Float64}}, Dict{Int64,Vector{Float64}}}=nothing)
    @.model(x,p) = p[1] + p[2]*exp(-x*p[3])
    ydata = en_i[t_in:t_fin]
    xdata = collect(1:length(ydata))
    par, _ = fit_routine(model, xdata, ydata, 3, wpm=wpm)
    println("\nPar[1] is ", par[1] )
    return par[1]
end
function gevp_dec(mat_obs::Vector{MatInfo}, mass::Vector{uwreal}; t0::Int64=2, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing, pl::Bool=true, n::Int64=1, pseudo::Bool=true, wilson::Bool=false)
    y0s = getfield.(mat_obs, :y0)
    mu_list = getfield.(mat_obs, :mu) 
    plat_dec0 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list)[y0s[i]+2:end-1] #matrices to diagonalise for a given sector [i]
        evecs = getall_eig(mat, t0)
        elem = mat_elem(evecs, mat, mass[i], n) #ground and excited state energy
        plat = select_plateau(mat_obs[i].ensinfo, mu_list, path_plat_dec)[i] .- y0s[i]
        try
            if pseudo
                plat_dec0[i] = dec(elem, plat, mass[i], mu_list[i], pl=pl, wilson=wilson)
            else
                plat_dec0[i] = vec_dec(elem, plat, mass[i], pl=pl)
            end
        catch
            println("Decay constant for the ensemble ", mat_obs[i].ensinfo.id, " failed in the sector ", mu_list[i], " pseudo sector?", pseudo)
            println("The associated effective mass is ", mass[i])
            plat_dec0[i] = uwreal(0)
        end
    end
    return plat_dec0
end
#this second function accepts plateau
function gevp_dec(mat_obs::Vector{MatInfo}, mass::Vector{uwreal}, plat::Vector{Vector{Int64}}; t0::Int64=2, wpm::Union{Nothing, Dict{Int64,Vector{Float64}}}=nothing, pl::Bool=true, n::Int64=1, pseudo::Bool=true, wilson::Bool=false)
    y0s = getfield.(mat_obs, :y0)
    mu_list = getfield.(mat_obs, :mu) 
    plat_dec0 = Vector{uwreal}(undef, length(mat_obs))
    for i in 1:length(mat_obs)
        mat = getfield(mat_obs[i], :mat_list)[y0s[i]+2:end-1] #matrices to diagonalise for a given sector [i]
        evecs = getall_eig(mat, t0)
        elem = mat_elem(evecs, mat, mass[i], n) #ground and excited state energy
        try
            if pseudo
                plat_dec0[i] = -dec(elem, plat[i], mass[i], mu_list[i], pl=pl, wilson=wilson)
            else
                plat_dec0[i] = -vec_dec(elem, plat[i], mass[i], pl=pl)
            end
        catch
            println("Decay constant for the ensemble ", mat_obs[i].ensinfo.id, " failed in the sector ", mu_list[i], " pseudo sector?", pseudo)
            println("The associated effective mass is ", mass[i])
            plat_dec0[i] = uwreal(0)
        end
    end
    return plat_dec0
end
function getall_eig(a::Vector{Matrix{uwreal}}, t0; iter=10)
    n = length(a)
    res = Vector{Matrix{uwreal}}(undef, n)
    [res[i] = uweigvecs(a[i], a[t0]) for i=1:n]
    return res 
end
function mat_elem(evec::Vector{Matrix{uwreal}}, ct_mat::Vector{Matrix{uwreal}}, mass::uwreal,  n::Int64)
    t = length(evec)
    res = Vector{uwreal}(undef, t)
    for i in 1:t
        aux = uwdot(evec[i][:,n], uwdot(ct_mat[i], evec[i][:,n]))[1]
        aux2 = 1 / (aux^2)^(0.25)   * exp(mass * (i)/2)
        res[i] = aux2 * uwdot(evec[i][:,n], ct_mat[i][:,n])
    end
    return res
end
function dec(mat_elem::Vector{uwreal}, plat::Array{Int64}, mass::uwreal, mu::Vector{Float64}; pl::Bool=true, wilson::Bool=false)
    aux = plat_av(mat_elem, plat)
    if pl
        uwerr(aux)
        x = 1:length(mat_elem)
        y = value.(mat_elem)
        dy = err.(mat_elem)
        v = value(aux)
        e = err(aux)
        figure()
        errorbar(x, y, yerr=dy, fmt="*", color="red")
        fill_between(plat[1]:plat[2], v-e, v+e, color="green")    
        ylim(v - 10*e, v + 10*e)
        display(gcf())
        close()
    end
    if !wilson
        return sqrt(2/ (mass)^3) * sum(mu)* aux
    else
        println("ciao its wilson")
        return sqrt(2/mass) * aux 
    end
end
function vec_dec(mat_elem::Vector{uwreal}, plat::Array{Int64}, mass::uwreal; pl::Bool=true)
    aux = plat_av(mat_elem, plat)
    if pl
        uwerr(aux)
        x = 1:length(mat_elem)
        y = value.(mat_elem)
        dy = err.(mat_elem)
        v = value(aux)
        e = err(aux)
        figure()
        ylim(v - 10*e, v + 10*e)
        errorbar(x, y, yerr=dy, fmt="*", color="red")
        fill_between(plat[1]:plat[2], v-e, v+e, color="green")    
        display(gcf())
    end
    return sqrt(2 / mass) * aux #aux / mass
end
function param_av(fit_masses::Vector{uwreal})
    w = 1 ./ err.(fit_masses).^2
    av = sum(w .* fit_masses) / sum(w)
    return av  
end
function get_meff(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false, wpm::Union{Nothing, Dict{String,Vector{Float64}}}=nothing)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list)
    return meff.(obs, plat, pl=pl, wpm=wpm)
end
function get_f(obs::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo; pl::Bool=false)
    mu_list = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list)
    return dec_const_pcvc.(obs, plat, m, pl=pl)
end
function select_plateau(ensinfo::EnsInfo)
    ens = ensinfo.id
    f = readdlm(path_plat)
    head  = String.(f[:,1])
    delim = findall(x->occursin("#", x), head)
    edelim = findfirst(x->occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)
    del = delim[cdelim]+1 : delim[cdelim+1]-1
    head_ = String.(f[del, 1])
    plat_ = Int64.(f[del, 2:3])
    plat = Vector{Int64}(undef, 2)
    n = findfirst(x-> occursin("t0", x), head_)
    plat[1] = plat_[n,1]
    plat[2] = plat_[n,2]
    return plat
end
function select_plateau(ensinf::EnsInfo, mu_list, path_plat)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end

function match_muc(muh, m_lh, m_sh, target)
    M = (2/3 .* m_lh .+ 1/3 .* m_sh) 
    par, chi2exp = lin_fit(muh, M)
    muh_target = x_lin_fit(par, target)
    return muh_target
end
function match_muc_heavymass(muh, m_hh, target)
    par, chi2exp = lin_fit(muh, m_hh)
    muh_target = x_lin_fit(par, target)
    return muh_target
end
function match_phih(muh,phih, phih_ph)
    par, chi2exp = lin_fit(muh, phih)
    muh_target = x_lin_fit(par, phih_ph)
    
    return muh_target
end

function aic_weigth(cat::Cat)
    w_aux = exp.(-1/2 * getfield(cat, :aic))
    w_aux1=filter(x->x<10000, w_aux)
    index = findall(x->x<10000, w_aux)
    norm = sum(w_aux1)
    w = w_aux1 ./ norm
    return w, index
end
function aic_syst(cat::Cat)
    w,index = aic_weigth(cat)
    aux1 = sum( w .* getfield(cat,:fit_res)[index].^2)
    aux2 = sum( w .* getfield(cat,:fit_res)[index])^2
    return sqrt(abs(value(aux1 - aux2)))
end
function aic_model_ave(cat::Cat)
    w, index = aic_weigth(cat)
    res = sum( w.* getfield(cat, :fit_res)[index])
    uwerr(res)
    return res
end
function category_av(cat_arr::Vector{Cat})
    best_res = aic_model_ave.(cat_arr)
    syst = aic_syst.(cat_arr)
    tot_err = err.(best_res).^2 .+ syst.^2
    w_aux = 1 ./ (tot_err)
    w = w_aux ./ sum(w_aux)
    aux1 = sum( w .* best_res.^2)
    aux2 = sum( w .* best_res)^2
    av = sum( w .* best_res) 
    return av, sqrt(abs(value(aux1 - aux2)))
end

function syst_av(syst_err::Vector{Float64})
    aux = sum(syst_err.^2)
    return sqrt(aux)
end
function read_BDIO(path::String, ens_id::String)
    file = filter(x->occursin(ens_id, x), readdir(path, join=true))
    println(file)
    r = Vector{uwreal}(undef, 0)
    fb = BDIO_open(file[1], "r")
    BDIO_seek!(fb)
    push!(r, read_uwreal(fb))
    while BDIO_seek!(fb, 2)
        push!(r, read_uwreal(fb))
    end
    BDIO_close!(fb)
    return r
end
